<?
/*
 * Class: AbstractGalmetaPostPlugin
 * The abstract class that all GalmetaPost plugins are required to extend to be a part
 * of the GalmetaPost System
 * 
 * Author: 
 *  Galmeta <contact at galmeta dot com>
 *  Bryan English <bryan at bluelinecity dot com>
 */
class AbstractGalmetaPostPlugin extends Hookable {
    
    /*
     * Property: galmetapost
     * Reference to <GalmetaPostRequest> object created for the current request. This
     * is set in  <__construct()>
     */    
    var $galmetapost;
    
    /*
     * Property: name
     * String, Common name of the plugin.
     */
    var $name;
    
    /*
     * Property: description
     * String, Description of what the plugin does.
     */
    var $description;
    var $version;
    var $author;
    var $usage;
    var $link;
    var $copyright;
    var $template_data = array();
    
    /*
     * Method: __construct
     * <AbstractGalmetaPostPlugin> constructor. Set's the <GalmetaPostRequest> reference, 
     * looks at self and creates an id based on the name of the class, (ie, GalmetaPostPlugin_include
     * becomes "include"), checks to see if it <isInstalled()> or <isOld()> then call appropriate 
     * methods to <install()> or <update()>.
     * 
     * Parameters:
     *  GalmetaPostRequest - A reference to the current GalmetaPostRequest created in the
     *  beginning
     */
    function AbstractGalmetaPostPlugin ( &$GalmetaPostRequest )
    {       
        $this->galmetapost =& $GalmetaPostRequest;
        $this->id = preg_replace('/^GalmetaPostPlugin_?/i','',get_class($this));

        if (!$this->isInstalled())
        {         
            $this->install();
        } 
        elseif ($this->isOld())
        {
            $this->update();
        }
        
        $this->initializeHookPoints();
        
    }

    /*
     * Method: install
     * Does the nessecary things to install this plugin. This should always be called
     * in the derieved class if you plan on overriding it. It installs the plugin's current
     * version, installation timestamp using <registry()>and registers uninstallation 
     * information using <registerUninstall()>.  
     */
    function install ()
    {
        $this->registry ( 'version', $this->version );
    	$this->registry ( 'installed', date("F j, Y, g:i a") );
        $this->registerUninstall ( 'all', 'plugins/' . $this->id . '//' );
    }
    
    /*
     * Method: update
     * Does the nessecary things to update this plugin. This should always be called
     * in the derieved class if you plan on overriding it. It updates the plugin's current
     * version, update timestamp using <registry()>and registers uninstallation 
     * information using <registerUninstall()>.  
     */
	function update ()
    {
		$this->registry ( 'version', $this->version );
		$this->registry ( 'installed', date("F j, Y, g:i a") );
        $this->registerUninstall ( 'all', 'plugins/' . $this->id . '//' );
    }

    /*
     * Method: isInstalled
     * Checks to see if this plugin is installed by checking the <registry>
     * 
     * Return:
     *  Boolean - true if installed, false if not.
     */
    function isInstalled ()
    {
		return ( $this->registry ( 'installed' ) !== null );
    }

    /*
     * Method: isOld
     * Checks to see if this plugin is newer than the one registered in the <registry>
     * 
     * Return:
     *  Boolean - true if old, false if not.
     */
    function isOld ()
    {
		return ( floatval($this->registry ( 'version' )) < floatval($this->version) );
    }

    /*
        Method: initializeHookPoints
        This function creates a series of hookpoints in the GalmetaPostRequest object using the hooks pattern so
        plugins can insert their own processing at various points. Plugins can insert their own functions via
        the <AbstractGalmetaPostPlugin::registerPluginHook()> function, or using <GalmetaPostRequest::registry()> function.
        This queries the registry database for all names matching 'hooks/%' and hooks based on record content.
        
    */    
    function initializeHookPoints(){

        $this->hook( 'run' );
        $this->hook( 'admin' );
        $this->hook( 'install' );
        $this->hook( 'update' );

        //load registered hooks//
        $hooks = $this->galmetapost->registry('hooks/'. $this->id .'//', null);

        if ( $hooks )
        {
            foreach ( $hooks as $hook )
            {
                $specs = explode( '/', $hook['name'] );
                $this->hook ( $hook['content'] , $specs[3], $specs[2] );
            }       
        }
    }

    /*
     * Method: run
     * Method that is called by the GalmetaPost system when parsing for plugins in content
     * such as {pluginname:}. Anything returned by this function then replaces the plugin
     * keyword in the content.
     * 
     * Parameters:
     *  params - String, the attribute list found in the plugin command that represent the
     *  parameters to the plugin.
     *  context - GalmetaPostResource, a galmetapost resource represening the executing 
     *  container.
     * 
     * Return:
     *  string - The HTML to replace the location of that plugin on the resource.
     */
    function run (){
    }

    /*
     * Method: canAccess
     * Checks to see if the current user has rights to execute this plugin.
     * 
     * Return:
     *  boolean - true if they have execute rights, false if they don't
     */
    function canAccess()
    {
        return $this->galmetapost->call( 'checkAccess', '{'. $this->id . ':}','x');
    }
    
    /*
     * Method: registerRewrite
     * Registers a url rewriting hook in the registry. Similar to mod_rewrite but
     * on the software level. Places it into the "rewrites/" branch of the registry.
     * 
     * Parameters:
     *  pattern - Regular Expression pattern to match against the resource url. Please
     *  note that this pattern is only applied to the resource portion of the parsedURI
     *  (see <parse_uri()>)
     *  replace - If the pattern matches, this is what replaces the resource url.
     *  If the replacement results in another url with a query string ( myurl?a=1&b=2 )
     *  then the query string portion is automatically placed by the <GalmetaPostResource>
     *  object into the <GalmetaPostResource::rewrite_vars> array.
     * 
     * See Also:
     *  <GalmetaPostRequest::registerRewrite()>
     *  <GalmetaPostResource::rewrote>
     */
    function registerRewrite( $pattern, $replace )
    {
        return $this->galmetapost->registerRewrite( $pattern, $replace );
    }
    
    /*
     * Method: registerPluginHook
     * Registers a plugin's method to a specific hook using the hook pattern. Use this
     * to install a handler for certain form POST's, etc. This function also automatically
     * registers an uninstallation key for anything registered, see <registerUninstall()>.
     * 
     * Parameters:
     *  function - String, the plugin method to bind to the hook point.
     *  hook - String, the hook point at which you want to bind the method too. For a
     *  list of hook points, see <GalmetaPostRequest::initializeHookpoints()>. To reference
     *  an external object method, qualify it with "objectname/methodname" 
     *  where - Optional String, a way to control when this function is called
     *  upon execution of the hook. This defaults to "at" but possible values include:
     *  ( before | at | after ) in that order and "around" which places the function in
     *  both "before" and "after".
     *  order - Optional Integer, defaults to 5 and is just another level of priority
     *  which can be assigned within the individual "where" keywords above. 
     * 
     * See Also:
     *  <GalmetaPostRequest::_hooks>
     *  <GalmetaPostRequest::hook()>
     *  <GalmetaPostRequest::run()>
     */
    function registerPluginHook( $function, $hook, $where = 'at', $order = 5 )
    {
        $this->galmetapost->registerHook ( $this->id . '::' . $function, $hook, $where, $order );

        $uname = $hook . '/' . $where . '/' . $order . '/' . $this->id . '::' . $function;

        $this->registerUninstall ( $uname, 'hooks/' . $uname );
    }

    /*
     * Method: registerUninstall
     * Creates an uninstallation note location to use in a registry clean up in the event
     * this plugin is removed.
     * 
     * Parameters:
     *  name - String, Some unique name, relative to this plugin.
     *  uname - String, A single registry path to delete when this plugin is deleted.
     * 
     * Example:
     * After running the include plugin, it should create an uninstallation entry like so:
     * > plugins/include/uninstall/all = plugins/include//
     * 
     * The double slash is a special convention in the registry to recursively select
     * all keys in that branch. See <GalmetaPostRequest::registry()> for more info. 
     */    
    function registerUninstall ( $name, $uname )
    {
        $this->galmetapost->registry('plugins/' . $this->id . '/uninstall/' . $name, $uname );
    }

    /*
        Method: registry
        Plugin-level registry access function. Shortcut to the plugin's registry name space ie
        myPlugin would add "key" to "plugins/myPlugin/key". 

        Parameters:
            name - Name of registry file to create
            value - Optional, Value to assign to registry entry.
            def - Optional, returned if registry key doesn't exist.
        
        Return:
            mixed - if value is null, returns the value of the requsted registry.
    */
    function registry ( $name, $value = null, $def = null )
    {
		return $this->galmetapost->registry ( 'plugins/' . $this->id . '/' . $name, $value, $def);
    }

  
    /*
     * Method: getParameters
     * This method takes a single string parameter containing name="value" pairs and parses
     * them into an associative array. However some sanitation is applied. It looks at the
     * plugins <usage> property for matching "name"s and only includes ones that are specified
     * in <usage>. If a name="value" is missing when compared against <usage>, then the 
     * 'default' value is used from the <usage> array.
     * 
     * Parameters:
     *  str - String of name="value" pairs to parse, similar to html attributes.
     * 
     * Return:
     *  array - An associative array of the allowed name="value" pairs.
     * 
     * Example:
     * > getParameters( '  list="index" display="red" silent="true"  ' );
     * > -- returns --
     * > array( 'list'=>'index', 'silent'=>true ) // and display was not used because it's
     * >                                          // not in the <usage> array.
     */
    function getParameters( $str ){
    
        $params = array();
      
        if ( is_string($str) )
        {
            //set defaults
            if ( isset( $this->usage ) && is_array( $this->usage ) )
            {
                foreach ( $this->usage as $key => $value )
                {
                    $params[$key] = $value['default'];
                }
            }
            else
            {
                return $params;
            }
            
            //trim the params to 1K//
            $str = trim(substr(trim($str),0,1024));

            //html decode passed str and parse out into name="value" pairs//
            $matches = array();
            preg_match_all('/([a-zA-Z0-9_]+)\s*\=\s*\"(.*?)\"/s',html_entity_decode($str), $matches );  
           
            //use only vars declared in <usage>//            
            foreach ( $params as $key => $value )
            {
                if ( ($r = array_search( $key, $matches[1] )) !== false )
                {
                    $fvalue = trim( $matches[2][$r] );
                    $params[$key] = ( $fvalue == 'true' || $fvalue == 'false' ? ( $fvalue == 'true' ? true : false ) : $fvalue );
                }
            }
                   
            return $params;
       } 
       else 
       {
           return $str;
       }
    }

    /*
     * Method: set
     * General setter function for adding vars to be used by a php template for this plugin. 
     * Simply adds an element to the <template_data> assocative array.
     * 
     * Parameters:
     *  name - Name of the variable to set.
     *  value - Value to set the variable to.
     * 
     * Return:
     *  Boolean - True on success.
     * 
     * See Also:
     *  <renderTemplate()> 
     *  <get()> 
     */
    function set ( $name, $value )
    {
        $this->template_data[$name] = $value;
        return true;
    }

    /*
     * Method: get
     * General getter function for getting vars previously set by <set()>
     * 
     * Parameters:
     *  name - Name of the variable to get.
     *  default - Value to return if said variable doesn't exist.
     * 
     * Return:
     *  mixed - The value found in <template_data> or default if not found.
     * 
     * See Also:
     *  <renderTemplate()> 
     *  <set()> 
     */
    function get ( $name, $default = null)
    {
        return (isset($this->template_data[$name])?$this->template_data[$name]:$default);
    }
    
    /*
     * Method: renderTemplate
     * Loads a php template (*.thtml), executes it and returns the results. After trying out
     * countless templating systems, smarty, tpl, phptal, etc I realized php templates when done
     * well, for sake of simplicity, still out-ranks these.
     * 
     * Parameters:
     *  0..n - All the parameters are optional. If no parameters are supplied, the method
     *  will auto load a template in the same directory named 'index.thtml'. 
     * 
     *  If one parameter is passed, the method assumes it is the name of the template to load, 
     *  sans the .thtml extension. 
     * 
     *  If two parameters are found, the first is assumed to be an additional assocative arrray
     *  of template vars to pass. The second is assumed to be the template name to use.
     * 
     * Return:
     *  String - Contents of the included template which are parsed by the php engine, or
     *  an error if the template is not found.      
     */
    function renderTemplate ( ) {

        $data = $this->template_data;

        switch (func_num_args())
        {
            case 0:
                $template = 'index';
            break;
            case 1:
                $template = func_get_arg(0);
            break;
            case 2:
                $data = array_merge( func_get_arg(0), $data );
                $template = func_get_arg(1);
            break;
            default:
                $template = 'index';
            break;
        }

        //sanitize template. only allow alphanumerics. No crazy stuff//
        $template = preg_replace( '/[^0-9a-z_]/i', '', $template );

        foreach ( $data as $_name => $_value )
        {
            $$_name = $_value;
        }

        //check for template//
        if ( file_exists( $this->galmetapost->getVar('path') . '_ext/plugins/'. $this->id .'/'. $template .'.thtml' ) )
        {
            ob_start();
            include ( $this->galmetapost->getVar('path') . '_ext/plugins/'. $this->id .'/'. $template .'.thtml');
            return ob_get_clean();
        } 
        else 
        {
            return 'Template ('. $template .') not Found';
        }
    }
    
    /*
     *  Method: getPlugin
     *  Retrieves a plugin from the cached internal array or the plugins folder, method
     *  override in Hookable
     * 
     *  Parameters:
     *      plugin - String, name of the plugin to get.
     * 
     *  Return:
     *      string - Reference to the loaded plugin or false if not found.
     */
    function &getPlugin( $plugin )
    {
        return $this->galmetapost->getPlugin( $plugin );
    }       

    /*
     * Method: uninstallPlugin
     * Override method for Hookable class to perform any potential cleanup
     * if a registered hook results in a call to a non-existant object, in this
     * case it means the plugin has been uninstalled.
     * 
     * Parameters:
     *  name - String, name of the plugin to uninstall.
     */
     function uninstallPlugin( $name )
     {
        $this->galmetapost->uninstallPlugin( $name );                                    
     }    
}

?>