<?php
$GALMETAPOST_VERSION = '0.2';

/*
 * Class: GalmetaPostRequest
 * This class is the primary class of all Galmeta Post. There should be only one instance
 * of this class created for each browser request. Once created, it's based among the
 * various <GalmetaPostResource>, <AbstractGalmetaPostPlugin> objects throughout the system.
 * This contains most of the resource access code, the hooking API, and much more. The
 * particular resource (page) that was requested by the user will always be set in 
 * <GalmetaPostRequest::request> as a <GalmetaPostResource> object.
 * 
 * This class also acts as a -time manager for any plugins that are needed by the request.
 * 
 * Package: 
 *  Galmeta Post
 * 
 * Author: 
 *  Galmeta <contact at galmeta dot com>
 *  Bryan English <bryan at bluelinecity dot com>
 * 
 * Download: 
 *     <http://galmeta.com/post>
 *
 * License:
 *     Galmeta Post is a Open Source Content Management System
 *     Copyright (C) 2007  Galmeta LLC
 *     
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License
 *     as published by the Free Software Foundation; either version 2
 *     of the License, or (at your option) any later version.
 *     
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *     
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class GalmetaPostRequest extends Hookable {

    /*
        Property: version
        Version of this galmetapost installation
    */
    var $version;
    
    /*
        Property: name
        Name of this installion of galmetapost. This is the default resource.
    */
    var $name = '';

    /*
        Property: description
        A short description of the web site, this used for rss/atom feeds and in other places.
    */
    var $description = '';

    /*
        Property: registry_name    
        Name of the registry database table
    */
    var $registry_name;

    /*
        Property: email    
        Email address of galmetapost administrator. 
    */
    var $email;

    /*
        Property: table_name
        Used for mulitiple galmetapost installations to seperate databases
    */
    var $table_name;

    /*
        Property: files_path
        Full system path to a world writeable files directory to upload resources
    */
    var $files_path;

    /*
        Property: cache_path
        Full system path to a world writeable files directory for cached objects
    */
    var $cache_path;

    /*
        Property: url
        Full web url to this galmetapost installation
    */
    var $url;

    /*
        Property: path
        Full system path to this galmetapost installation
    */
    var $path;

    /*
        Property: request
        <GalmetaPostResource> object of the user requested resource. This will always be set since any interaction with the 
        web site would require a request of some sort.
    */
    var $request;   

    function setRequest( &$r ){ $this->request =& $r; }
    function &getRequest( ){ return $this->request; }

    /*
        Property: remoteuser
        Remote user ip for use when timestamping edited pages.
    */
    var $remoteuser;

    /*
        Property: lock_timeout
        Time to expire a locked page in minutes
    */
    var $lock_timeout;

    /*
        Property: login_failures
        How many login failures to allow before locking up
    */
    var $login_failures;

    /*
        Property: access_denied_message
        Access denied message
    */
    var $access_denied_message = '<span class="error {$name}">Access Denied to {$name} resource!</span>';

    /*
        Property: auth_check
        Disables/enabled multiuser and access control, almost always true. Set to false if using httpauth to control access.
    */
    var $auth_check = true;

    /*
        Property: safehtml
        Flag indicating to use safehtml parser to strip potentially dangerous html markup from POSTed data
    */
    var $safehtml = true;

    /*
        Property: date_format
        Format string for dates using php syntax. Defaults to 'F j, Y (g:i a)'
    */
    var $date_format;

    /*
        Property: use_mod_rewrite
        Boolean flag indicating the use of mod_rewrite, effects the <url_prefix> string.
    */
    var $use_mod_rewrite = true;

    /*
        Property: url_prefix
        url prefix incase of no mod_rewrite. Blank if using mod_rewrite, '?' otherwise.
    */
    var $url_prefix;

    /*
        Property: core_override
        Boolean flag controlling core override feature, allows multiple core/css resources in
        the folder hierarchy.
    */
    var $core_override = false;

    /*
        Property: versioning
        Boolean flag used to toggle resource versioning. 
    */
    var $versioning = false;

    /*
        Property: _conn
        Private. Database handle
    */
    var $_conn;

    /*
        Property: _loaded_plugins
        Private. Array of loaded plugins
    */
    var $_loaded_plugins = array();

    /*
        Property: _loaded_rules
        Private. Array of loaded time rules
    */
    var $_loaded_rules = NULL;

    /*
        Property: _access_tokens
        Privite. Loaded access tokens for current user
    */
    var $_access_tokens = NULL;

    /*
        Property: _html_includes
        Private HTML Includes js,css,etc.
    */
    var $_html_includes = array('js'=>array(),'css'=>array(),'rss'=>array(),'atom'=>array(),'rss2'=>array());

    /*
        Method: GalmetaPostRequest
        Initializes the GalmetaPostRequest Object. Loads configuration variables, sets defaults, initialize hookpoints
        connect to database, handle ajax requests.

        Parameters:
            $conf - Configuration hash to initilize <GalmetaPostRequest> object. Taken from config.php
    */
    function GalmetaPostRequest( &$conn, $conf ){

        $this->version = $GLOBALS['GALMETAPOST_VERSION'];

        //defaults//
        $this->date_format = 'F j, Y (g:i a)';
        $this->lock_timeout = 2;
        $this->login_failures = 5;
        $this->path = dirname($_SERVER['SCRIPT_FILENAME']) . '/';
        $this->name = 'GalmetaPost';
        $this->history_length = 10;
        $this->access_plugin = 'access';

        //load and apply configuration array//
        if (is_array($conf))
        {
            foreach ($conf as $key => $value)
            {
                $this->$key = $value;
            }         
        } 
        else 
        {
            return false;
        }

        //set time variables//
        $this->remoteuser = $_SERVER['REMOTE_ADDR'];

        $this->url_prefix = ($this->use_mod_rewrite?'':'index.php?');

        $this->db =& $conn;
        
        //load access plugin or failsafe default//
        if ( !$this->getPlugin( $this->access_plugin ) )
        {
            $this->hook('defaultCheckAccess','at','checkAccess');
            $this->hook('defaultGetMethodAccess','at','getMethodAccess');
        }

        //create hookpoints//
        $this->initializeHookPoints();
        $this->call('init');

        //load theme if enabled//
        if ($this->user_themes && $_COOKIE['theme'])
        {
            $this->theme = substr(preg_replace('/[^a-zA-Z0-9_]/','',$_COOKIE['theme']),0,255);
        }

        //check for ajax call//
        $matches = array();
        if ( !empty( $_GET['AJAX_CALL'] ) && preg_match('/^(?:([a-z0-9_]+)\/)?([a-z0-9_]+)$/i',$_GET['AJAX_CALL'],$matches) )
        {
            if (file_exists("_ext/ajax/{$matches[2]}.php"))
            {
                include_once "_ext/ajax/{$matches[2]}/{$matches[2]}.php";
            } 
            elseif (file_exists("_ext/plugins/{$matches[1]}/ajax/{$matches[2]}.php")) 
            {
            	include_once "_ext/plugins/{$matches[1]}/ajax/{$matches[2]}.php";	
        	} 
            else 
            {
                echo "Invalid Ajax Call";
            }
            exit();
        }
    }

    /*
     *  Method: initializeHookPoints
     *  This function creates a series of hookpoints in the GalmetaPostRequest object using the hooks pattern so
     *  plugins can insert their own processing at various points. Plugins can insert their own functions via
     *  the <AbstractGalmetaPostPlugin::registerPluginHook()> function, or using <GalmetaPostRequest::registry()> function.
     *  This queries the registry database for all names matching 'hooks/%' and hooks based on record content.
     *  
     *  Hook Points:
     *      insertResource - at <insertResource()>
     *      updateResource - at <updateResource()>
     *      deleteResource - at <deleteResource()>
     *      moveResource - at <moveResource()>
     *      copyResource - at <copyResource()>
     *      saveVersion - at <saveVersion()>
     *      findByName - at <findByName()>
     *      findAllByName - at <findAllByName()>
     *      getCount - at <getCount()>
     *      locateCore - at <locateCore()>
     *      getPlugin - at <getPlugin()>
     *      rules - at <rules()>
     *      HTMLInclude - at <HTMLInclude()>
     *      postDispatch - blank, called on <index.php> 
     *      checkAccess - blank, usually set to plugin "access/checkAccess"
     *      getMethodAccess - blank, usually set to plugin "access/getMethodAccess"
     *      getUserSetting - blank, usually set to plugin "user/getUserSetting"
     */
    function initializeHookPoints(){

        $this->hook( 'insertResource' );
        $this->hook( 'updateResource' );
        $this->hook( 'deleteResource' );
        $this->hook( 'moveResource' );
        $this->hook( 'copyResource' );
        $this->hook( 'saveVersion' );
        $this->hook( 'findByName' );
        $this->hook( 'findAllByName' );
        $this->hook( 'findAllByContent' );
        $this->hook( 'getCount' );
        $this->hook( 'locateCore' );
        $this->hook( 'getPlugin' );
        $this->hook( 'rules' );
        $this->hook( 'lock' );
        $this->hook( 'unlock' );
        $this->hook( 'HTMLInclude' );
        $this->hook( 'viewDispatch' );

        //load registered hooks//
        $hooks = $this->registry('hooks/GalmetaPostRequest//', null);
 
        if ( $hooks )
        {
            foreach ( $hooks as $hook )
            {
                $specs = explode( '/', $hook['name'] );
                $this->hook ( $hook['content'] , $specs[3], $specs[2] );
            }       
        }
    }

    /*
     *   Method: postDispatch
     *   Empty method used as a hookpoint for plugins processing POSTed data.
     *   If called it will simply call the hookpoint "postDispatch"
     * 
     *  Parameters
     *      action - value of $_POST['do'] which specifies what to do. 
     */
    function postDispatch( $action )
    {
        $this->call( 'postDispatch', $action ); 
    }
  
    /*
     * Method: viewDispatch
     * Handle the displaying of the request using the optional view modifier
     * 
     * Parameters:
     *  view - Optional, string forcing a particular view instead of using whats
     *  in the original request. See <GalmetaPostResource:view>.
     */
    function viewDispatch( $view = null )
    {
        $view = epick( $view, $this->request->getVar('view'), 'normal' );
         
        switch ( $view )
        {
            case "ping":
                echo ($this->request->exists()?1:0);
            break;
            
            case "count":
                echo $this->request->getCount();
            break;
            
            case "raw":
                if (preg_match('/(^|\/)css/',$this->request->getVar('name')))
                {
                    header("Content-Type: text/css");
                }
                echo $this->request->get();
            break;
            
            case "only":
                $content = $this->request->getRender();
                $this->HTMLInclude( $this->locateCore( 'css' ), 'css' );
                echo $this->HTMLInclude() . $content;
            break;
        
            case "rdf":
            case "rss":
            case "pod":
                $data = $this->request->getList();
                $title = $this->getVar('name');
                $description = $this->getVar('description');
                $link = $this->getVar('url');
                header("Content-Type: application/rss+xml");
                if ( $view == 'pod' )
                {
                    include 'pod.php';
                }
                else if ( $this->request->getVar('parameters') == '2' )
                {
                    include 'rss2.php';        
                }
                else
                {
                    include 'rdf.php';        
                }                
            break;
        
            case "atom":
                $data = $this->request->getList();
                $title = $this->getVar('name');
                $description = $this->getVar('description');
                $link = $this->getVar('url');
                header("Content-Type: application/atom+xml");        
                include 'atom.php';
            break;
        
            case "text":
                echo strip_tags($this->request->getRender());
            break;
        
            case "print":
                $content = $this->request->getRender();
                $title = $this->request->getVar('title');        
                include "print.php";
            break;
        
            case "xml":
                if ( $_SERVER['QUERY_STRING'] == '!xml')
                {
                    $xml = $this->findAllByName("%");
                }
                elseif ( $this->request->getVar('list') )
                {
                    $xml = $this->request->getList();
                }
                else
                {
                    $xml = array(array(
                        'name' => $this->request->getVar('resource'),
                        'content' => $this->request->getRender(),
                        'edited_on' => $this->request->getVar('edited_on'),
                        'edited_by' => $this->request->getVar('edited_by'),
                        'version' => $this->request->getVar('version')
                    ));
                }
                
                header("Content-Type: text/xml");
                include 'xml.php';
            break;
        
            case "json":
                include 'json.php';  
            break;
        
            case "v":
            case "history":
            case "normal":
            case "admin":
            default:
                $core = new GalmetaPostResource( $this, $this->locateCore());
                $content = $core->getRender();
                $request =& $this->getRequest();
                $title = $request->getVar('title');
                include "normal.php";
            break;
        } 
    }
  
    /*
     *  Method: findByName
     *  DB access method. Finds a resource by name. This function does not
     *  check access or sanitize the name variable. It assumes it's been done already.
     *  This function has no security.
     * 
     *  Parameters:
     *      name - Name of the galmetapost page to check for
     *     
     *  Return:
     *      mixed - False when doesn't exist, array of select results otherwise...
     *          array(
     *              'id' => int
     *              'name' => string
     *              'content' => string
     *              'edited_on' => timestamp
     *              'edited_by' => string
     *              'locked' => timestamp
     *              );
     */
    function findByName ( $name )
    {
        $sql = "SELECT * FROM `{$this->table_name}` WHERE name = '". slash( $name ) ."'";   
        $result = $this->db->Execute( $sql ) or 
            die(_lang("Unable to query the database. Please  install.php."));

        if ( $result->RecordCount() != 0 )
        {
            return $result->fields;
        } 
        else 
        {
            return false;
        }
    }

    /*
     *  Method: findAllByName
     *  DB access method. Query the database for a list of pages matching the keyword path.
     *  
     *  Parameters:
     *      path - The path of the galmetapost content to query for with trailing slash.
     *      sort - Field to sort on. Defaults to edited_on.
     *      order - Order to sort by. Defaults to DESC.
     *      get_versions - Get versions. Defaults to false.
     *      offset - Record offset of the list to start at. Defaults to 0.
     *      limit - Record limit of the list. Defaults to 30.
     *      checkaccess - Remove records not accessible by user. Defaults to true
     *      core_resources - Include system files in query. Defaults to false
     *  
     *  Return:
     *      Array of Associative arrays of the results of the query. (name, content, edited, edited_ip)
     */
    function findAllByName ( $path, $sort = null, $order = null, $get_versions = false, $offset = null, $limit = null, $checkaccess = true, $core_resources = false )
    {
        if ( $get_versions )
        {
            $path .= '!v%';
        }
        elseif ( strpos($path,'%') === false )
        {
            $path .= '%';
        }
        
        $sql =  "SELECT * FROM `{$this->table_name}` WHERE name LIKE " . $this->db->qstr($path);

        if ( !$get_versions )
        {
            $sql .= " AND name NOT LIKE '%!v%' "; 
        }

        if ( !$core_resources )
        {
            $sql .= " AND name NOT REGEXP '(^|\/)(core|css|page_not_found|access_denied)' ";
        }

        $sql .= " ORDER BY " . addslashes(($sort?$sort:'edited_on')) . ' ' . addslashes(($order?$order:'DESC'));   

        if ( $limit !== null )
        {
            $offset = intval($offset);
            $limit = intval($limit);
            $sql .= " LIMIT " . ($offset<0?0:$offset) . ',' . ($limit<0?30:$limit);
        }

        $rs = $this->db->Execute( $sql ) or die( $this->db->ErrorMsg() . ' in ' . __FILE__);

        $list = array();

        if ( $rs->RecordCount() > 0 )
        {
            //check access rights for each resource//
            while ( !$rs->EOF() )
            {                
                if ( (!$checkaccess) || $this->checkAccess( $rs->Fields('name'), 'r' ) )
                {
                    $list[] = $rs->fields;
                }
                $rs->MoveNext();
            }
            return $list;
        } 
        else 
        {
            return false;
        }
    }

    /*
     *  Method: findAllByContent
     *  DB access method. Query the database for a list of pages matching the keyword path.
     *  
     *  Parameters:
     *      search - The search string to match against
     *      path - The path of the galmetapost content to query for with trailing slash.
     *      sort - Field to sort on. Defaults to edited_on.
     *      order - Order to sort by. Defaults to DESC.
     *      get_versions - Get versions. Defaults to false.
     *      offset - Record offset of the list to start at. Defaults to 0.
     *      limit - Record limit of the list. Defaults to 30.
     *      checkaccess - Remove records not accessible by user. Defaults to true
     *      core_resources - Include system files in query. Defaults to false
     *  
     *  Return:
     *      Array of Associative arrays of the results of the query. (name, content, edited, edited_ip)
     */
    function findAllByContent ( $search, $path = null, $sort = null, $order = null, $get_versions = false, $offset = null, $limit = null, $checkaccess = true, $core_resources = false )
    {
        $sql  = "SELECT * FROM `{$this->table_name}` WHERE ";

        if ( $path )
        {
            if ( $get_versions )
            {
                $path .= '!v%';
            }
            elseif ( strpos($path,'%') === false )
            {
                $path .= '%';
            }
            
            $sql .= "name LIKE " . $this->db->qstr($path) . " AND ";
        }

        if ( !$get_versions )
        {
            $sql .= "name NOT LIKE '%!v%' AND ";
        }

        if ( !$core_resources )
        {
            $sql .= " name NOT REGEXP '(^|\/)(core|css|page_not_found|access_denied)' AND ";
        }

        $sql .= " MATCH content AGAINST (". $this->db->qstr($search) .")";

        $sql .= " ORDER BY " . addslashes(($sort?$sort:'edited_on')) . ' ' . addslashes(($order?$order:'DESC'));   

        if ( $limit !== null )
        {
            $offset = intval($offset);
            $limit = intval($limit);
            $sql .= " LIMIT " . ($offset<0?0:$offset) . ',' . ($limit<0?30:$limit);
        }

        $total = $this->db->Execute( $sql ) or die($this->db->ErrorMsg());
        
        $total_rows = $total->RecordCount();            
        
        $data = array();            
        if ( $total_rows > 0 ) 
        {
            while ( !$total->EOF )
            {
                if ( (!$checkaccess) || $this->call('checkAccess', $total->Fields('name'),'r'))
                {
                    $data[] = $total->fields;
                }
                $total->MoveNext();
            }
            
            return $data;
        }
        
        return false;
    }

    /*
        Method: getCount
        Query the database for a count of resources matching the passed path. Not the most efficient
        function since it uses <findAllByName()> to get the count.

        Parameters:
            path - The name of the galmetapost content to query for with trailing slash.
            get_versions - Get versions. Defaults to false.
            checkaccess - Use access control to get count. defaults to true.
            
        Return:
            int - count of resources matching path            
     */
    function getCount( $path, $get_versions = false, $checkaccess = true, $core_resources = false )       
    {
        $ls = $this->call( 'findAllByName', $path, null, null, $get_versions, null, null, $checkaccess, $core_resources );
        return ($ls === false ? 0 : count($ls));
    }

    /*
        Method: getVar
        General var getter for GalmetaPostRequest

        Parameters: 
            var - name of the property to get
            def - default value to return if none-existant

        Return
            mixed - value of $this->var
    */
    function getVar( $var, $def = null )
    {
        if ( !isset($this->{$var}) )
        {
            trigger_error(__CLASS__ . '::' . htmlspecialchars($var) . ' Undefined',E_USER_ERROR);
        }
        else if ( $this->{$var} !== null )
        {
            return $this->{$var};
        }
        else
        {
            return $def;
        }
    }

    /*
     *  Method: locateCore
     *  Finds a core resource (core,css,page_not_found,access_denied), taking into account possible local core
     *  resource overrides. This employes a cascading method of choosing a core
     *  resource. It will look at the currently requested resource and find
     *  the closest core|css objects looking for themed first, then the default root
     *  core and css. The theme is selected from, in this order, $GALMETAPOSTCONFIG['theme'] then
     *  <GalmetaRequest::registry('plugins/theme')> then <GalmetaRequest::getUserSetting('theme')>
     *
     *  Parameters:            
     *      type - Name of the resource to look for (core|css|page_not_found|access_denied). Defaults to core.
     *
     *  Returns:
     *      string - Full resource name of the closest core to use given the context.
     *      if resource not found, returns blank string. If the core is not found, issues
     *      an error.
     *
     *  See Also:
     *      <HTMLInclude()>
     *      <theme>
     *
     *  Example:
     *      A request for /services/childcare would initate a location process looking 
     *      in order for "/services/childcare/core" then "/services/core" then "/core"
     *      and would return the longest string, in essence the closest core.
     * 
     *  Core Order of Importance:
     *      - Themed core page closest to requested resource
     *      - Core page closest to requested resource
     */
    function locateCore( $type = 'core' ){

        //if core, automatically bundle the css page//
        if ( $type == 'core' )
        {            
            $this->HTMLInclude( $this->url_prefix . $this->locateCore('css') . '!raw', 'css' );
        }

        if ( $this->core_override )
        {
            //generate places to look//
            $look = array();
            $core = '';
            $request =& $this->getRequest();            
            $parts = explode( '/', $request->getVar('resource') );
            $current_themes = array();
            
            //look for currently active themes//
            $themes = $this->call('getCurrentThemes');

            foreach ( $parts as $part )
            {
                //add the current theme to core request//
                if ( $themes[$core.$part] )
                {                    
                    $look[] = $core . $part . '/' . $type . $themes[$core.$part];
                }                
                
                $core .= $part . '/';
                $look[] = $core . $type;                
            }
            
        }
        
        //base core & theme//      
        $look[] = $type;
        if ( $themes['_'] )
        {
            $look[] = $type . $themes['_'];
        }        

        $sql = "SELECT LENGTH(name) as size, name, content FROM `{$this->table_name}` WHERE name IN ('";
        $sql .= implode("','",$look) . "')";

        if ( preg_match('/(^|\/)core/',$type) )
        {
            $sql .= "AND content LIKE '%{include:}%'";   	  
        }

        $sql .= " ORDER BY size";

        $result = $this->db->Execute( $sql );

        if ( !$result )
        {
            die("Unable to query the database. Please  install.php.");
        }

        if ( $result->RecordCount() == 0 )
        {
            if ( !preg_match('/(^|\/)core/',$type) )
    	    {
               return '';
       	    }
            else
            {
               die("Unable to locate the core resource. Please verify a {include:} plugin is in the core resource.");
            }
        }

        //check access rights for each resource//
        $scope = 0;
        while ( !$result->EOF() )
        {            
            if ( $this->checkAccess( $result->Fields('name'), 'r' ) )
            { 
                $ccnt = count_chars($result->Fields('name'));
                $scnt = $ccnt[ord('/')];
                if ( $scope <= $scnt )
                {
                    $scope = $scnt;
                    $type = $result->Fields('name');
                }
            }
            $result->MoveNext();
        }

        return $type;
    }

    /*
        Method: checkAccess
        Check the access of a given user. This is used simply as a hookpoint holder.

        Parameters:
            resource - Resource being requested to check against
            method - What they plan to do to the resource [ w - write[cud] r - read x - execute c - create u - update d - delete ]
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']
        
        Return:
            Boolean - True if access is granted, false if otherwise.

        See Also:
            <GalmetaPostPlugin_access>
            <call()>
            <hook()>
    */
    function checkAccess( $resource, $method, $username = null )
    {
        return $this->call( 'checkAccess', $resource, $method, $username );
    }

    /*
        Method: defaultCheckAccess
        Check the access of a given user.

        Parameters:
            resource - Resource being requested to check against
            method - What they plan to do to the resource [ w - write[cud] r - read x - execute c - create u - update d - delete ]
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']
        
        Return:
            Boolean - True if access is granted, false if otherwise.
    */
    function defaultCheckAccess( $resource, $method, $username = null )
    {
        $methods = $this->getMethodAccess( $resource, $username );

        $len = strlen( $method );

        for ( $cnt = 0; $cnt < $len; $cnt++ )
        {
            if ( !$methods[$method{$cnt}] )
            {
                return new HookBreakCondition( false );
            }
        }

        return new HookBreakCondition( true ); 
    }
    
    /*
        Method: getUserSetting
        Get a user setting from the registry. This is a hook function. ( /users/myUser/settings/aSetting )

        Parameters:
            setting - Named setting to retrieve
            default - Optional, if setting doesn't exist, this is what should be returned
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']
        
        Return:
            Mixed - The user setting retrieved from the registry or $default if setting undefined.

        See Also:
            <GalmetaPostPlugin_user>
            <()>
            <hook()>
    */
    function getUserSetting( $setting, $default = null, $username = null )
    {
        $setting = $this->call( 'getUserSetting', $setting, $default, $username );
        
        //incase no hook points created yet//
        if ( $setting === null )
        {
            $setting = $default;
        } 
        
        return $setting;
    }


    /*
        Method: getMethodAccess
        Get the access rights by method of a particular resource. This is simply a hookpoint holder.
    
        Parameters:
            resource - Resource being requested.
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']

        Return:
            array - Associative array of booleans with the rights. $rights['c'] = create, $rights['r'] = read,
            $rights['u'] = update, $rights['d'] = delete, $rights['x'] = execute.

        See Also:
            <GalmetaPostPlugin_access>
            <()>
            <hook()>
    */
    function getMethodAccess( $resource, $username = null )
    {
        return $this->call( 'getMethodAccess', $resource, $username );
    }

    /*
        Method: defaultGetMethodAccess
        Get the access rights by method of a particular resource.
    
        Parameters:
            resource - Resource being requested.
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']

        Return:
            array - Associative array of booleans with the rights. $rights['c'] = create, $rights['r'] = read,
            $rights['u'] = update, $rights['d'] = delete, $rights['x'] = execute.
    */
   function defaultGetMethodAccess( $resource, $username = null )
   {            
        if ( $username === null )
        {
            if ( !empty($_SESSION['username']) )
            {
                $username = $_SESSION['username'];
            } 
            else 
            {
                $username = 'guest';
            }
        }

        if ( $username != 'guest' && $this->registry('users/' . $username ) )
        {
            return new HookBreakCondition( array('c'=>true,'r'=>true,'u'=>true,'d'=>true,'x'=>true) );                
        }
        else
        {
            static $allowed_plugins = null;
            if ( !$allowed_plugins )
            { 
                $allowed_plugins = file($this->path . 'acl.txt');
                foreach( $allowed_plugins as $key => $value )
                {
                    $allowed_plugins[$key] = trim($value);
                }
            }
            
            if ( preg_match("/^admin\//",$resource) 
                || ( preg_match("/^\{[^}]+\:\}$/",$resource) 
                    && !in_array(substr($resource,1,strlen($resource)-3),$allowed_plugins) ))
            {
                return new HookBreakCondition( array('c'=>false,'r'=>false,'u'=>false,'d'=>false,'x'=>false) );
            }
            else
            {
                return new HookBreakCondition( array('c'=>false,'r'=>true,'u'=>false,'d'=>false,'x'=>true) );
            }
        }        
    }

    /*
        Method: getPlugin
        Retrieves a plugin from the cached internal array or the plugins folder

        Parameters:
            plugin - String, name of the plugin to get.

        Return:
            string - Reference to the loaded plugin or false if not found.
    */
   function &getPlugin( $plugin )
   {
      //if not already loaded and cached//
      if ( !$this->_loaded_plugins[$plugin] )
      { 
         if ( file_exists( $this->path . '_ext/plugins/'.$plugin.'/'.$plugin.'.php') )
         {
            include_once ( $this->path . '_ext/plugins/'.$plugin.'/'.$plugin.'.php');
            if ( class_exists ('GalmetaPostPlugin_' . $plugin) )
            {
                $class = 'GalmetaPostPlugin_' . $plugin;                
                $this->_loaded_plugins[$plugin] =& new $class( $this );
            } 
            else 
            {
                return false;
            }
         } 
         else 
         {
            return false;
         }
      }

      return $this->_loaded_plugins[$plugin];   
   }


    /*
        Method: rules
        Loads and returns the content rewrite rules in the _ext/rules directory and
        plugin specific rules found in _ext/plugins/* /rules
        
        Return:
            array - Array of the loaded rules. array( array(//patterns//), array(//replacements//) )
    */
   function rules()
   {
      $pattern = array();
      $replace = array();

      //load additional rule files//
      if ($this->_loaded_rules == null )
      {
         $ls1 = glob($this->path . '_ext/rules/*/*.php');
         $ls2 = glob($this->path . '_ext/plugins/*/rules/*.php');
         
         $rules = array_merge( ($ls1?$ls1:array()), ($ls2?$ls2:array()) );
         sort($rules);
         foreach ($rules as $import)
         {
            include ($import);
         }
        
         $this->_loaded_rules = array($pattern,$replace);
      }

      return $this->_loaded_rules;
   }


    /*
    * Method: saveVersion
    * Saves a version of the named resource resource by adding a new record with the
    * version info appended to the name. Parameters must be stripped of slashes.
    * 
    * Example:
    *   A version 1 page named "content" is saved as "content!v1".
    * 
    * Parameters:
    *   name - Full name of the resource to save a version of.    
    * 
    * Return:
    *   mixed - false if not versioning, else version number of the resource.
    */
    function saveVersion($name)
    {
        //check versioning access and if installation allows it//
        if ( $this->versioning )
        {
            $old = $this->db->Execute("SELECT CONCAT(name,'!v',version) as name, content, edited_on, edited_by, version FROM `{$this->table_name}` WHERE name=". $this->db->qstr($name,QSTR)) or die($this->db->ErrorMsg());
            $old_row = $old->GetRows(1);
            $this->db->Execute("INSERT INTO `{$this->table_name}` (name, content, edited_on, edited_by, version) VALUES ("
            . $this->db->qstr($old_row[0]['name']) .","
            . $this->db->qstr($old_row[0]['content']) .","
            . $this->db->qstr($old_row[0]['edited_on']) .","
            . $this->db->qstr($old_row[0]['edited_by']) .","
            . $this->db->qstr($old_row[0]['version']) .")") or die(__LINE__ .' '.$this->db->ErrorMsg());
            
            return $old_row[0]['version'];
        }
        else
        {
            return false;
        }
    }


    /**
     *  Method: registry
     *  This method is the only way to access the registry settings for GalmetaPost. It's a
     *  getter/setter combo function. It's sanitized but not access controlled. ''DO NOT
     *  ALLOW USER SUBMITTED VALUES INTO THIS FUNCTION'' The parameters must be stripped 
     *  of any addslash addititives.
     *
     *  Parameters:
     *      name - Name of the registry entry to set/get
     *      value - Optional, Value to set the registry to. Blank string deletes key. 
     *      def - Default value to return if registry name doesn't exist
     *
     *  Return:
     *      mixed - value of requested registry key, or <def> if not defined.
     */
    function registry( $name, $value = null, $def = null )
    {

        $ls = ( $name{(strlen($name)-1)} == '/' ); // users/ will just return the first level
        $dump = ( $name{(strlen($name)-2)} == '/' );  // users// will return everything in the namespace

        if ( $dump )
        {
            $name = substr( $name, 0, strlen($name)-1 );
        }

        $nameslashstar = $this->db->qstr( $name .'%/%' );
        $namestar = $this->db->qstr( $name . '%' );
        $name = $this->db->qstr( $name );

        $where = "name" . ( $ls ? " LIKE {$namestar}" : " = {$name}");
        $where .= ( $dump ? '' : " AND name NOT LIKE {$nameslashstar}");

        $sql = "SELECT name, content FROM `{$this->registry_name}` WHERE " . $where . " ORDER BY name";

        $rs = $this->db->Execute( $sql ); 

        if ( $value === null )
        {
            //get the registry value at $name//
            if ( $rs->RecordCount() > 0 )
            {
                if ( $ls )
                {
                    $result = array();
                    while ( !$rs->EOF() )
                    {
                        $result[] = $rs->fields;
                        $rs->MoveNext();
                    }
                }
                else
                {
                    $row = $rs->GetRows(1);         
                    $result = $row[0]['content'];                
                }
            } 
            else 
            {
                $result = $def;
            }
        } 
        else 
        {   
            //modify the registry value//
            if ( $value == '' )
            {
                $this->db->Execute("DELETE FROM `{$this->registry_name}` WHERE " . $where ) or die(__LINE__ . ' ' . $this->db->ErrorMsg());
            }
            else
            {
                if ( $rs->RecordCount() > 0 )
                {
                    $this->db->Execute("UPDATE `{$this->registry_name}` SET name={$name}, content=". $this->db->qstr($value) ." WHERE " . $where ) or die(__LINE__ . ' ' . $this->db->ErrorMsg());
                } 
                else 
                {
                    $this->db->Execute("INSERT INTO `{$this->registry_name}` (name, content) VALUES ({$name},". $this->db->qstr($value) .")" ) or die(__LINE__ . ' ' . $this->db->ErrorMsg());
                }
            }

            $result = true;
        }

        return $result;
    }

    /*
        Method: registerHook
        This is a shortcut to the registry function specifically for adding function hooks into the GalmetaPost cms.
    
        Parameters:
            function_uri - String, Name of the function to call on the hook. For Plugins it takes the form "plugin/methodname"
            hook - Hook point to attached the function to.
            where - Optional, where to attach the function [before|at|after|around], defaults to 'at'.
            order - Optional, used to control the priority of the function, defaults to '5'

        Return:
            mixed - result of registry change
    */
    function registerHook( $function_uri, $hook, $where = 'at', $order = 5 )
    {
        $this->registry( 'hooks/' . $hook . '/' . $where . '/' . $order . '/' . $function_uri, $function_uri );
    }

    /*
        Method: registerRewrite
        This is a shortcut to the registry function specifically for adding url rewrites. Not implemented yet.
    
        Parameters:
            pattern - matching pattern to apply to the url
            replace - what to replace the request with if pattern matches

    */    
    function registerRewrite( $pattern, $replace )
    {
        $this->registry( 'rewrites/' . $pattern, $replace );
    }

    /*
     * Method: moveResource
     * Move a resource in the database preserving meta data. Not access controlled. Same as
     * renaming. Parameters must be stripped of slashes.
     *
     * Parameters:
     *  oldname - Current Name of the resource to move
     *  newname - New Name of the resource.
     * 
     * Return:
     *  mixed - result of the db query
     */
    function moveResource ( $oldname, $newname )
    {
        return $this->db->Execute("UPDATE `{$this->table_name}` SET name="
            . $this->db->qstr($newname) ." WHERE name="
            . $this->db->qstr($oldname)) or die(__LINE__.' '.$this->db->ErrorMsg());
    }

    /*
     * Method: copyResource
     * Copy a resource in the database preserving meta data. Not access controlled.
     * Parameters must be stripped of slashes.
     * 
     * Parameters:
     *  oldname - Current Name of the resource to copy
     *  newname - New Name copy of the resource.
     * 
     * Return:
     *  mixed - result of the db query
     */
    function copyResource ( $oldname, $newname )
    {
        return $this->db->Execute("INSERT INTO `{$this->table_name}` (`name`,`content`,`edited_on`,`edited_by`,`locked`,`version`) SELECT "
            . $this->db->qstr($newname) .", content, edited_on, edited_by, locked, version FROM `{$this->table_name}` WHERE name="
            . $this->db->qstr($oldname)) or die(__LINE__.' '.$this->db->ErrorMsg());
    }
    
    /*
     * Method: deleteResource
     * Delete a resource from the database. Not access controlled. 
     * Parameters must be stripped of slashes.
     *
     * Parameters:
     *  name - Name of the resource to delete
     * 
     * Return:
     *  mixed - result of the db query
     */
    function deleteResource ($name)
    {
        return $this->db->Execute("DELETE FROM `{$this->table_name}` WHERE name="
            . $this->db->qstr($name) );
    }        

    /*
     * Method: insertResource
     * Insert a resource into the database. Not access controlled. Parameters must be stripped of slashes.
     *
     * Parameters:
     *  name - Name of the resource to insert
     *  content - Content of the resource
     *  user - Username of the user doing the insert.
     * 
     * Return:
     *  mixed - result of the mysql query
     */
    function insertResource ($name,$content,$user)
    {     
        return $this->db->Execute("INSERT INTO `{$this->table_name}` (name, content, edited_on, edited_by) VALUES ("
            . $this->db->qstr($name) .","
            . $this->db->qstr($content) .",NOW(),"
            . $this->db->qstr($user . '@' . $this->remoteuser) .")" ) or die(__LINE__.' '.$this->db->ErrorMsg());          
    }

    /*
    * Method: updateResource
    * Update a resource in the database. Not access controlled. Parameters must be stripped of slashes.
    *
    * Parameters:
    *   name - Name of the resource to update
    *   content - Content of the resource
    *   user - Username of the user doing the update.
    * 
    * Return:
    *   mixed - result of the mysql query
    */
    function updateResource ($name,$content,$user)
    {

        return $this->db->Execute("UPDATE `{$this->table_name}` SET name="
            . $this->db->qstr($name) .", content=". $this->db->qstr($content) 
            .", edited_on=NOW(), edited_by=". $this->db->qstr($user . '@' . $this->remoteuser) 
            .", locked = 0, version = (version+1) WHERE name=" 
            . $this->db->qstr($name)) or die(__LINE__.' '.$this->db->ErrorMsg());
    }
    
    /**
     * Query the database for a unique id using the id field.
     *
     * @return int
     */
    function getAutoId ()
    {     
        $result = $this->db->Execute( "SELECT CONV(MAX(id)+1,10,36) as auto FROM `{$this->table_name}`" ) or die($this->db->ErrorMsg());
        $row = $result->GetRows(1);
        return $row[0]['auto'];
    }

    /*
    *  Method: getAccessDenied
    *  Access function for the standard access denied message.
    * 
    *  Parameters:
    *     name - name of the resource access is denied too.
    * 
    * Return:
    *     string - access denied string.
    */
    function getAccessDenied( $name = '')
    {        
        if ( $name != 'access_denied' )
        {
            $ac =  $this->locateCore( 'access_denied' );
            if ( $ac )
            {
                $denied =& new GalmetaPostResource( $this, $ac );
                return $denied->getRender();
            }
        }
        
        $name = htmlspecialchars( $name );
        return preg_replace( '/\{\$name\}/', $name, $this->access_denied_message );
    }

    /*
     * Method: HTMLInclude
     * Adds/Gets an html level include such as a JS file or CSS file. If no paramters are 
     * provided, assumes get and returns the equivilant html for including files.
     *
     * Parameters:
     *  file - Optional, URL Path to the resource.
     *  force - Optional, Force a type of resource [css|js] incase of non-typical extension.
     * 
     * Return:
     *  mixed - true on add, string of imploded includes if no parameters given.
     */
    function HTMLInclude( $file = null, $force = null )
    {
        $map = array(
            'js' =>  array('match'=>'/\.js$/','a'=>'<script type="text/javascript" src="','b'=>'"></script>' . "\n"),
            'css' => array('match'=>'/\.css$/','a'=>'<link type="text/css" rel="stylesheet" href="','b'=>'" />' . "\n"),
            'rss' => array('match'=>'/\!rss$/','a'=>'<link title="'. $this->name .' RSS 0.1 Feed" type="text/xml" rel="alternate" href="','b'=>'" />' . "\n"),
            'rss2' => array('match'=>'/\!rss2$/','a'=>'<link title="'. $this->name .' RSS 2.0 Feed" type="application/rss+xml" rel="alternate" href="','b'=>'" />' . "\n"),
            'atom' => array('match'=>'/\!atom$/','a'=>'<link title="'. $this->name .' Atom Feed" type="application/atom+xml" rel="alternate" href="','b'=>'" />' . "\n")
            );

        $content = '';

        foreach ( $map as $type => $info )
        {
            if ( $file )
            {
                if ( (preg_match($info['match'],$file) || $force == $type) && !in_array($file,$this->_html_includes[$type]) )
                {
                    $this->_html_includes[$type][] = $file;
                    return true;
                }
            }
            else
            {
                if ( count( $this->_html_includes[$type] ) != 0 )
                {
                    $content .= $info['a'] . implode($info['b'].$info['a'],$this->_html_includes[$type]) . $info['b'];
                }
            }
        }

        return $content;
    }

   /**
    * Method: lock
    * Locks a resource by timestamping the locked column. This function
    * is not access controlled but the input is quoted 
    *
    * Parameters:
    *   resource - full path of the galmetapost resource to lock
    */
    function lock( $resource )
    {
        $this->db->Execute("UPDATE `". $this->table_name ."` SET locked = '". date('Y-m-d H:i:s') ."' WHERE name=". $this->db->qstr($resource,QSTR)) or die($this->db->ErrorMsg());         
    }

   /**
    * Method: unlock
    * Unlocks a resource by nullifying the column. This function
    * is secure and the request is sanitized.
    * This function is secure.
    * 
    * Parameters:
    *   resource - full path of the galmetapost resource to unlock
    */
    function unlock( $resource )
    {
        $this->db->Execute("UPDATE `". $this->table_name ."` SET locked = 0 WHERE name=". $this->db->qstr($resource,QSTR) ) or die($this->db->ErrorMsg());         
    }

    /*
     * Method: uninstallPlugin
     * Override method for Hookable class to perform any potential cleanup
     * if a registered hook results in a call to a non-existant object, in this
     * case it means the plugin has been uninstalled.
     * 
     * Parameters:
     *  name - String, name of the plugin to uninstall.
     */
     function uninstallPlugin( $name )
     {
        $uninstall = $this->registry('plugins/' . $name . '/uninstall//');
        foreach ( $uninstall as $row )
        {
           $this->registry( $row['content'], '' );
        }                                    
     }
     
   /*
    * Method: redirect     
    * Sends a redirect reqeust to the browser using the header() function.
    * 
    * Parameters:
    *     path - Path to redirect to. If it no protocol is found will assume
    *     http:// plus $this->url.
    */
    function redirect( $path )
    {
        $path = trim($path,"\n\r");

        if ( !preg_match('/^https?\:\/\//', $path) )
        {
            $path = $this->url . $path;
        }
        
        header('Location: ' . $path );
        exit();
    }
}

?>