<?php
/*
 * Class: Hookable
 * Hookable is an abstract class that is meant to be extended by any class wanting
 * the ability to bind its own methods to other external object methods. It works
 * by maintaining an array of before and after method refrences to call in response
 * to one call via the call() method. This class almost can be thought of as creating
 * soft methods, where the parameters and return values can be subjected to external
 * plugin methods. As well as act as a trigger for other processes.
 */
class Hookable 
{
    /*
     *  Property: _hooks
     *  Private Array Hook points, array of method references. Might be overridden as static.
     * 
     *  Example:
     *  array (
     *      'mySoftFunction' => array( 
     *          'before' => array('plugin::mybeforefunction'),
     *          'at' => array('myatfunction'),
     *          'after' => array('plugin::myafterfunction'),
     *      )
     *  )
     *
     *  See Also:
     *      <run()>
     *      <hook()>
     */
    var $_hooks = array();    

    /*
     * Method: hook
     * Creates a hook point using the <_hook> property. Depending on the number of
     * arguments passed, will catalog a hook in <_hooks> and place the location and priority
     * of the hook.
     * 
     * Parameters:
     *  Method - String, Name of the method to bind. When used as only parameter, will create a hookpoint by the same name.
     *  In a addition, a foreign method can be referenced using the scope operator, "class::method". FOr this to work your
     *  extending object has to override <getPlugin()> to handle object resolution.
     *  Location - String, Where to hook the method [ before | at | after ]. Defaults to 'at'.
     *  Hook - String, name of the hookpoint, or softfunction. Defaults to Function.
     */
    function hook ( $method, $location = null, $hook = null ) 
    {    
        $location = pick( $location, 'at' );
        $hook = pick( $hook, $method );
        
        if ( !$this->_hooks[$hook] )
        {
            $this->_hooks[$hook] = array();
        }    
        
        if ( !$this->_hooks[$hook][$location] )
        {
            $this->_hooks[$hook][$location] = array();            
        }
        
        $this->_hooks[$hook][$location][] = $method;
    }

    /*
     * Method: hookIsEmpty
     * Checks a hook method and returns whether or not it is emtpy.
     * 
     * Parameters:
     *  hook - Hook name to check
     * 
     * Return
     *  boolean - true if hook is empty, false otherwise.
     */
    function hookIsEmpty( $hook )
    {
        
        return ( empty($this->_hooks[$hook]) || (count($this->_hooks[$hook]) == 0));
    }

    /*
    * Method: call
    * Calls a constructed hook point, taking into account all hooked in method references.
    *
    * Parameters:
    *   arg1 - name of the hookpoint to run.
    *   arg2..n - parameters to pass to the hookpoint functions
    * 
    * Return:
    *   mixed - The last non-null value returned from the hooked functions. 
    * 
    * Additional Info:
    *   Each function will be called in the hookpoint tree. Every function executed has the
    *   option to return a special case <HookCondition> object such as <HookPassCondition> or <HookBreakCondition>. 
    *   If a method call returns a <HookBreakCondition> the object's constructor parameter is immediatly returned by 
    *   <run()> and exited. If a call returns a <HookPassCondition> then the object's constructor parameters
    *   are used as the parameter override for the next call in the tree instead of the originally
    *   passed parameters. This allows a function to filter and change paramters for the next
    *   one to pickup. Please note that only methods hooked as "before" and "at" can override paramters. 
    *   Methods that are hooked as "after" accept only one parameter, the return value in the process. This allows
    *   for methods to filter the returning value.
    * 
    * See Also:
    *   <hook()>
    *   <Hookable::_hooks>
    */
    function &call (){                        
        $params = func_get_args();
        $call = array_shift($params);
        $pass = '';
        $return = null;
        $rtn = null;
        
        $pcnt = count($params);
        if ( $pcnt > 0 )
        {
            $pass = '$params[0]';
            for ( $c=1; $c < $pcnt; $c++ )
            {
                $pass .= ',$params['. $c .']';
            }
        }

        $eval_pass = array(
            'before' => $pass,
            'at' => $pass,
            'after' => '$return'
        );

        //process before and at hooks (parameter filter)//
        foreach( array('before','at','after') as $part)
        {
            $hook = $this->_hooks[$call][$part];
            if ( is_array($hook) )
            {
                foreach( $hook as $f ) 
                {
                    $res =& $this->hookResolve( $f );
                    
                    if ( $res['object'] )
                    {
                        eval("\$rtn = \$res['object']->{$res['method']}({$eval_pass[$part]});");
                    }
                    else 
                    {
                        $this->uninstallPlugin( $f );
                    }

                    //stop processing hooks if hook break detected//
                    if ( $rtn !== null )
                    {
                        if ( is_object($rtn) && is_subclass_of($rtn,'HookCondition') )
                        {
                            $rtn->handle( $params, $return );
                            if ( $rtn->isBreak() )
                            {
                                return $return;
                            }
                        }
                        else
                        {
                            $return = $rtn;
                        }
                    }                                            
                } 
            }
        }

        return $return;
    }
    
    /*
     * Method: hookResolve
     * This function handles resolving the correct object for a method call. It examines
     * the reference for a scope operator "::" and if it has one, calls <getPlugin()> to 
     * get the proper object. Otherwise it returns itself ($this) as the call will be a local
     * one.
     * 
     * Parameters:
     *  name - String, name of the hooked method.
     * 
     * Return:
     *  Array - Associative array containing the object instance ($foo['object']) and method name ($foo['method']). 
     */    
    function &hookResolve( $name )
    {
        $split = explode('::',$name);
        $res = array();
        if ( empty($split[1]))
        {
            $res['object'] =& $this;
            $res['method'] = $split[0];
        }
        else
        {
            $res['object'] =& $this->getPlugin( $split[0] );
            $res['method'] = $split[1];            
        }
        
        return $res;
    }
    
    /*
     * Method: getPlugin
     * A method meant to be overridden by the extending class to assist the hookResolve in retreiving
     * foreign object instances.
     * 
     * Parameters:
     *  name - String, class name of the object to get.
     * 
     * Return:
     *  mixed - Object if requested class object is found, null otherwise. 
     */
    function &getPlugin( $name )
    {
        //override me
        return null;
    }

    /*
     * Method: uninstallPlugin
     * A method meant to be overridden by the extending class to assist in cleaning up hook references
     * if the case is of a recently uninstalled plugin
     * 
     * Parameters:
     *  name - String, class name of the object to uninstall.
     */
    function uninstallPlugin( $name )
    {
        //override me
        return null;
    }

}

?>