<?php
/*
 * Class: GalmetaPostPlugin_access
 * This is a core plugin for Galmeta Post and provides access control support
 * for the whole cms. It uses an ACL method similar to *nix systems based on
 * groups, users and five methods of access; Create, Read, Update, Delete and
 * Execute, each which can be granted allow or deny.
 * 
 * All of this information is stored in the registry key named 'access' in a 
 * JSON block. This JSON block is read by the external library, JSON_Services
 * and compiled to form a given users's access rules. 
 * 
 */
class GalmetaPostPlugin_access extends AbstractGalmetaPostPlugin {

    function GalmetaPostPlugin_access( &$n )
    {
        $this->name = 'Access';
        $this->version = '1.0';
        $this->description = "Basic Access control plugin. Provides an Admin interface which configures user access using a JSON block and the voorhees control library. It's based of DAC security model.";
        $this->author = 'Bryan English <bryan@galmeta.com>';
        $this->link = 'http://galmeta.com/post/';
        $this->copyright = 'Copyright &copy; 2007 Galmeta LLC';
        
        parent::AbstractGalmetaPostPlugin( $n );
    }
    
    function install()
    {
        //install default access permissions for first non-guest user//
        $users = $this->galmetapost->registry("users//");

        foreach ( $users as $user )
        {
            if ( !preg_match('/^users\/guest/',$user['name']) )
            {                 
                $username = preg_replace('/^users\/([^\/]+)/','$1',$user['name']);
                $result = $this->galmetapost->registry('access',str_replace('$$$ADMIN_USER$$$',$username,implode(file($this->galmetapost->getVar('path') . '_ext/plugins/access/default.acl'))));
                break;
            }
        }
        
        $this->registerPluginHook('processEdit','GalmetaPostRequest/postDispatch');
        $this->registerPluginHook('checkAccess','GalmetaPostRequest/checkAccess');
        $this->registerPluginHook('getMethodAccess','GalmetaPostRequest/getMethodAccess');
        parent::install();
    }

    function update()
    {
        $this->install();
    }

    /*
     * Method: run
     * Run the plugin. Getting content to either read or edit.
     *
     * Return:
     *  String - rendered content
     */
     function run() {

        $schema = implode(file($this->galmetapost->getVar('path') . '_ext/plugins/access/access.voorhees'));
        $content = '';        

        $request =& $this->galmetapost->getRequest();

        if ( $request->getVar('resource') == 'admin/access' && $this->canAccess() )
        {        
            $this->galmetapost->HTMLInclude( '_lib/prototype.js' );
            $this->galmetapost->HTMLInclude( '_lib/voorhees/voorhees.css' );
            $this->galmetapost->HTMLInclude( '_lib/voorhees/voorhees.js' );
        
            $this->set('schema',$schema);
            $this->set('rights',$this->galmetapost->registry('access'));
        }

        return $this->renderTemplate();
    }

    /*
     * Method: processEdit
     * Save an access post
     */
    function processEdit( $action )
    {
        if ( $action == 'Update Access' )
        {
            $request =& $this->galmetapost->getRequest();
            if ( $request->getVar('resource') == 'admin/access' && $this->canAccess() ){
                $this->galmetapost->registry('access', deslash($_POST['access']));
            }
        }
    }

    /*
        Method: _getTokens
        Private, Extract tokens from the ACL JSON block

        Parameters:
            groups - Source JSON block object
            username - The username to isolate rules for.

        Return:
            Array - Array of access tokens for that user.
            The token is an object of the form:
            >
            > {'resource'=>'String', 
            >  'method'=>'crudx*', 
            >  'access'=>'allow|deny'} 
            >             
    */
    function _getTokens( $groups, $username ){
        $tokens = array();

        foreach ( $groups as $group )
        {            
            $child_tokens = array();
            //process subgroups//
            if ( isset($group->groups) )
            {
                $child_tokens = $this->_getTokens($group->groups,$username);
            }

            //grab only groups & rules that username is part of//
            if ( ($result = $this->_userInGroup($group,$username)) || count($child_tokens) > 0 )
            {
                //add group rules to tokens//
                if ( is_array($group->rules) )
                {
                    $tokens = array_merge($tokens,$group->rules);
                }

                //if user has specific rules, add them too//
                if ( is_array($result) )
                {
                    $tokens = array_merge($tokens,$result);
                }

                //if any subgroups had relavent tokens add them too//
                if ( $child_tokens )
                {
                    $tokens = array_merge($tokens,$child_tokens);
                }
            }
        }

        return $tokens;
    }

    /*
     * Method: _userInGroup
     * Check if username is in the passed JSON parsed Group object. If the username is
     * part of the group, return true or an array of that user's specific rules.
     * 
     * JSON Schema:
     *  'group':{
     *      'users':[
     *          {'user':'user1'},
     *          {'user':'user2'}
     *              ],
     *      'rules':[
     *          {'access':'allow','resource':'page','method':'crudx'}
     *              ]}
     *
     * Parameters:
     *      group - ACL object
     *      username - Username to look for
     *  
     *  Return:
     *      boolean - True or array of user rules if found, false otherwise.
     */
    function _userInGroup($group,$username)
    {
        if ( isset($group->users) && is_array($group->users) )
        {
            foreach ( $group->users as $user ) 
            {
                if ( $user->user == $username )
                {
                    if ( is_array($user->rules) )
                    {
                        return $user->rules;
                    } 
                    else 
                    {
                        return true;
                    }
                }
            }
        }

        return false;
    }


    /*
        Method: checkAccess
        Check the access of a given user.

        Parameters:
            resource - Resource being requested to check against
            method - What they plan to do to the resource [ w - write[cud] r - read x - execute c - create u - update d - delete ]
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']
        
        Return:
            Boolean - True if access is granted, false if otherwise.
    */
    function checkAccess( $resource, $method, $username = null )
    {
        $methods = $this->galmetapost->call('getMethodAccess', $resource, $username );

        $len = strlen( $method );

        for ( $cnt = 0; $cnt < $len; $cnt++ )
        {
            if ( !$methods[$method{$cnt}] )
            {
                return new HookBreakCondition( false );
            }
        }

        return new HookBreakCondition( true ); 
    }

    /*
        Method: getMethodAccess
        Get the access rights by method of a particular resource.
    
        Parameters:
            resource - Resource being requested.
            username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']

        Return:
            array - Associative array of booleans with the rights. $rights['c'] = create, $rights['r'] = read,
            $rights['u'] = update, $rights['d'] = delete, $rights['x'] = execute.
    */
   function getMethodAccess( $resource, $username = null )
   {
      if ( $username === null )
      {
         if ( !empty($_SESSION['username']) )
         {
            $username = $_SESSION['username'];
         } 
         else 
         {
            $username = 'guest';
         }
      }

      if ($this->galmetapost->getVar('auth_check'))
      {
         $tokens = $this->getAccess($username);
         $methods = array('c'=>false,'r'=>false,'u'=>false,'d'=>false,'x'=>false);

         $allow = false;
         $toggle = array('allow'=>true,'deny'=>false);

         //foreach access token apply rules to the passed resource//
         foreach ($tokens as $token)
         {            
              if (
               ($token->resource == '*' && $resource{0} != '{' ) || 
                $token->resource == $resource || 
               (preg_match('/(\/|\/\*)$/', $token->resource ) && preg_match("/^". str_replace('/','\/',$token->resource) .".*/",$resource)) ||
               ($token->resource == '{*:}' && preg_match('/^\{[a-z0-9_]+\:\}$/i',$resource))
              ){                               

                 if ( $token->method == '*' ) //apply access to all methods//
                 {
                    $t = $toggle[$token->access];
                    $methods = array('c'=>$t,'r'=>$t,'u'=>$t,'d'=>$t,'x'=>$t);
                 }
                 else //apply access to methods specified//
                 {
                     $len = strlen( $token->method );
                     for ( $cnt = 0; $cnt < $len; $cnt++ )
                     {
                         $methods[$token->method{$cnt}] = $toggle[$token->access];
                     }
                 }
              }
         }
        
         return new HookBreakCondition( $methods );
      } 
      else 
      {      
         return new HookBreakCondition( array('c'=>true,'r'=>true,'u'=>true,'d'=>true,'x'=>true) );
      }
   }

    /*
     * Method: getAccess
     * Gets an array of access tokens for a user or loads it from the JSON resource. An access
     * token is an object of the form:
     * >
     * > {'resource'=>'String', 
     * >  'method'=>'crudx*', 
     * >  'access'=>'allow|deny'} 
     * >
     * 
     * Parameters:
     *  name - Username to get the access rights of.
     * 
     * Return:
     *   array - Array of token objects
     */
    function getAccess( $name )
    {
        static $short_cache = array();

        if ( $name == $_SESSION['username'] )
        {
            if ($_SESSION['access_tokens'] == null)
            {
                $tokens = $this->loadAccess( $name );
                $_SESSION['access_tokens'] = $tokens;
            }

            return $_SESSION['access_tokens'];
        } 
        else 
        {            
            if ( !isset($short_cache[$name]) )
            {
                $short_cache[$name] = $this->loadAccess( $name );
            }

            return $short_cache[$name];
        }
    }

    /*
     * Method: loadAccess
     * Loads, parses and returns the access resource from the registry using the JSON.php library,
     * filtering out all access rules except for rules applying to the passed username.
     * 
     * Parameters:
     *  username - Username to get the access rights of.
     * 
     * Return:
     *  Array - array of token object's for that username's account.
     * 
     * See Also:
     *  <_getTokens()>
     */
    function loadAccess( $username )
    {
        if ($result = $this->galmetapost->registry('access'))
        {
            require_once $this->galmetapost->getVar('path') . '_lib/JSON.php';
            $json = new Services_JSON();
            $acl = $json->decode($result);
            $tokens = $this->_getTokens($acl,$username);
            return $tokens;
        }   
     
        return null;
    }    

}
?>