<?php
/*
 * Class: GalmetaPostPlugin_include
 * A GalmetaPost core plugin. This is the most essential plugin for GalmetaPost. It
 * handles "include"ing content from the database. This plugin is required in the core
 * resource in order to display a page requested by a user. When used with no parameters,
 * as in "core"s case, {include:} will get replaced by the "content" requested by the user.
 * 
 * To put it another way, {include:} will use the resource auto-loaded into the 
 * <GalmetaPostRequest::request> object. {include: part="title"} will refer to the same
 * resource but grab the title for it. In this way, the core resource uses the {include:}
 * plugin to act as a template throught the site.
 *
 * Plugin Parameters:
 *  file - Absolute URL path to the content to include, or filename from _static/ folder
 *  part - Part of the current content to get. [content|title|name|version|edited_on|edited_by|link] Defaults to "content".
 *  silent - If access denied or resource is missing, don't alert it.
 *  view - Used with "link" part to specify a view [xml|rssprint|rss|text|only|rdf|atom|raw|edit|editraw|new], defaults to plain link to resource.
 *  text - When generating a link, this is an optional override for the link text. If it is set to a image path, the plugin will automatically create an img tag.        
 * 
 * Used User Settings:
 *  safehtml - If <processEdit> should sanitize content submission with safehtml library,
 *  defaults to true. This helps prevent XSS attacks.
 *  ToolbarSet - Used by FCKEditor to choose a Toolbar, defaults to 'Editor'
 *  can_upload - Used by FCKEditor to enable uploading files by a user, defaults to false.
 *  upload_dir - Used by FCKEditor to restrict the user to uploading to a specific directory,
 *  defaults to the path specified in <GalmetaPostRequest::files_path>. Any string in this
 *  setting is appended to the system files_path in <GalmetaPostRequest::files_path>. So
 *  changing it to "reports/" will restrict the user to uploading to "_files/reports"
 *
 * 
 * Hookpoints:
 *  editorPlugins - this hookpoint can be utilized by other plugins looking to link-in php code upon
 *  rendering the FCKEditor. Any functions hooked into this must be set "after" and must accept a single
 *  array parameter, append a path to the array, then return the modifed array.
 *  
 */
class GalmetaPostPlugin_include extends AbstractGalmetaPostPlugin {

    /*
     * Method: GalmetaPostPlugin_include
     * Constructor for include plugin. Configures the specs for the plugin.
     * 
     * Parameters:
     *  G - Reference to the <GalmetaPostRequest> object created for the current request.
     */
   function GalmetaPostPlugin_include( &$G ){

        $this->name = 'Include';
        $this->version = '1.1';
        $this->author = 'Bryan English <bryan@galmeta.com>';
        $this->link = 'http://galmeta.com/post/';
        $this->copyright = 'Copyright &copy; 2007 Galmeta LLC';
        $this->description = 'Includes other galmetapost resources as if it were part of the current page. This allows you to construct advanced pages from multiple includes. Essentially the Server-Side Includes (ssi) of galmetapost.';


        $this->usage = array(
            'file' => array(
              'default' => null,
              'description' => 'Absolute URL path to the content to include, or filename from _static/ folder'),

            'part' => array(
              'default' => 'content',
              'description' => 'Part of the current content to get. [content|title|name|version|edited_on|edited_by|link] Defaults to "content".'),

            'silent' => array(
              'default' => false,
              'description' => 'If access denied or resource is missing, don\'t alert it.'),

            'view' => array(
              'default' => null,
              'description' => 'Used with "link" part to specify a view [print|rss2|text|only|rdf|atom|raw|edit|editraw|new], defaults to plain link to resource.'),

            'text' => array(
              'default' => null,
              'description' => 'When generating a link, this is an optional override for the link text. If it is set to a image path, the plugin will automatically create an img tag.'),

            'height' => array(
              'default' => 500,
              'description' => 'Controls the size of the edit box.'),
            
            'toolbar' => array(
                'default' => null,
                'description' => 'Controls the toolbar used for the plugin.')

            );

        $this->user_settings = array(
            'safehtml' => array(
                'default' => 'true',
                'description' => 'Toggles XSS filter library on and off for all edits.'),
            
            'can_upload' => array(
                'default' => 'false',
                'description' => 'Controls if a user can upload files to the server.'),
                
            'ToolbarSet' => array(
                'default' => 'Editor',
                'description' => 'Sets the FCKeditor toolbar set to use for a user. Possible values [Admin|Editor|Basic]'),
                
            'upload_dir' => array(
                'default' => '',
                'description' => 'Restricts a user to another folder within the upload dir (_files/).')
        );

        $this->messages = array();

        parent::AbstractGalmetaPostPlugin( $G );
    }

    /*
     * Method: install
     * Registers <processEdit> to the postDispatch hook to handle content edit submissions. 
     */
    function install()
    {
        $this->registerPluginHook('processEdit','GalmetaPostRequest/postDispatch');
        $this->registerPluginHook('editorPlugins','include/editorPlugins');
        parent::install();
    }

    function update()
    {
        $this->install();
    }

    /*
     * Method: run
     * Run the include plugin. Getting content to either read or edit.
     *
     * Parameters:
     *  params - Plugin parameters passed from galmetapost resource or in-code call. See
     *  <usage> for more information on allowed parameters.
     *  resource - Resource context of this executing plugin.
     * 
     * Return:
     *  String - rendered content
     */
    function run( $params, &$resource ){

        $params = $this->getParameters( $params );
        $template = 'index';

        if ( preg_match('/[a-zA-Z0-9_-]+\.(html|php)/',$params['file']) && $params['part'] == 'content' )
        {
            if ( file_exists('_static/' . $params['file']) )
            {
                return implode(file('_static/' . $params['file']));
            }
            else
            {
                return 'File not found: _static/' . $params['file'];
            }
        } 
        else
        {            
            //if no file parameter assume initially requested resource//
            if ( empty($params['file']) )
            {
                $include =& $this->galmetapost->getRequest();
                $params['file'] = $include->getVar('uri');

                //if this is a request for content, then use core template since//
                //this is the main body include //
                if ( $params['part'] == 'content' && ( $include->getVar('view') == 'edit' 
                    || $include->getVar('view') == 'editraw' 
                    || $include->getVar('view') == 'new' ) )
                {
                    $template = 'core';
                }
            }
            else
            {
                //create new resource object to hold include request//
                $include =& new GalmetaPostResource( $this->galmetapost, $params['file'], $resource->getVar('resource') );
            }
            
            //die silently//
            if ( $params['silent'] && !$include->getVar('can_read') )
            {
                return '';
            }           

            switch ( $params['part'] ){               

                case 'content':
                    //check for a editing view//
                    if ( $include->getVar('view') == 'edit' 
                        || $include->getVar('view') == 'editraw' 
                        || $include->getVar('view') == 'new' )
                    {   
                        $raw = $include->get( 'content' );

                        //check permissions//
                        if ( ( $include->getVar('exists') && $include->getVar('can_update') ) 
                            || ( !$include->getVar('exists') && $include->getVar('can_create') ) )
                        {
                            static $AUTO = 0;

                            $editor = array();
                            
                            $editor['AUTO'] = $AUTO;
                            $editor['view'] = $include->getVar('view');
                            $editor['editor_plugins'] = '';
                            $editor['resource'] = $include->getVar('resource');
                            $editor['include_content'] = ( empty($_POST['content'][$AUTO]) ? $raw : deslash($_POST['content'][$AUTO]) );
                            $editor['messages'] = implode('<br />',$this->messages);
                            $editor['toolbar'] = pick($params['toolbar'],$this->galmetapost->getUserSetting('ToolbarSet','Editor'));
                            $editor['height'] = intval($params['height']);
                            $editor['url'] = $this->galmetapost->getVar('url');
                            $editor['css'] = $this->galmetapost->getVar('url') . 'css!raw';
                            
                            $this->galmetapost->call( 'HTMLInclude', '_lib/mootools.v1.1.js');
                            $this->galmetapost->call( 'HTMLInclude', '_ext/plugins/include/fckeditorstarter.js');
                            $this->galmetapost->call( 'HTMLInclude', '_lib/fckeditor/fckeditor.js');

                            $editor['fckeditor'] = ( strpos( $include->getVar('name'), 'css' ) === false 
                                                     && $include->getVar('view') != 'editraw' );
                            $path = $this->galmetapost->getVar('path');
                            $rules = $this->call('editorPlugins',array(
                                $path . '_ext/plugins/include/editor.captcha.php',
                                $path . '_ext/plugins/include/editor.guest.php',
                                $path . '_ext/plugins/include/editor.lock.php',
                                $path . '_ext/plugins/include/editor.save.php'                               
                            ));

                            if ( !empty($rules) )
                            {                           
                                foreach ($rules as $import)
                                {
    	                            ob_start();
    	                            include $import;
                                    $editor['editor_plugins'] .= ob_get_clean();
                                }
                            }
                                                
                            $content = $this->renderTemplate( $editor, 'edit' );

                            $AUTO++;
                        }
                        else //display Access Denied//
                        {
                            return $this->galmetapost->getAccessDenied();
                        }
                    }
                    elseif ( $include->getVar('view') == 'v' || $include->getVar('view') == 'history' )
                    {
                        $parameters = $include->getVar('parameters');
                        if ( empty($parameters) && ($ls = $this->galmetapost->call( 'getPlugin', 'ls')) )
                        {                            
                            $content = $ls->call( 'run', 'path="/' . $include->getVar('resource') . '" as="versions" sort="version" order="DESC" versions="true"', $resource );
                        }
                        else
                        {
                            $include->setVar('resource', $include->getVar('resource') . '!v' . intval( $parameters ) );
                            $content = $include->getRender();
                        }
                    }
                    elseif ( $include->getVar('view') == 'form' )
                    {                        
                        $include->get();
                        $include->setVar('content',preg_replace('/(\{include\:[^\}]+file\=\"[^\"]+)(\"[^\}]*\})/','$1!edit$2',html_entity_decode($include->getVar('content'))));
                        $content = $include->render();
                        $template = 'core';
                    }
                    else
                    {
                        $content = $include->getRender();
                    }
                break;

                case 'title':
                    $content = $include->getVar('title');
                break;

                case 'name':
		        case 'resource':
                case 'edited_on':
                case 'edited_by':
                case 'version':
                    $parameters = $include->getVar('parameters');
                    if ( ($include->getVar('view') == 'v' || $include->getVar('view') == 'history') && !empty($parameters) )
                    {                        
                        $include->setVar('resource','!v' . intval($parameters) );                        
                        $include->get();
                        $content = $include->getVar($params['part']);
                    }
                    else
                    {
                        $include->get();
                        $content = $include->getVar($params['part']);
                    }
                break;

                case 'link':
                
                    //sanitize view parameter//
                    if ( !preg_match('/^'. VALID_GALMETAPOST_VIEWS .'$/', $params['view'] ) )
                    {
                        $params['view'] = null;
                    }                    
                
                    //get link type from view param//
                    switch ( $params['view'] )
                    {
                        case 'editraw':                        
                        case 'edit':
                            $include->get();
        
                            //if don't have access, return nothing//
                            if ($this->galmetapost->getVar('auth_check') && ( $include->getVar('exists') 
                                && !$include->getVar('can_update') ) || ( !$include->getVar('exists') 
                                && !$include->getVar('can_create') ) )
                            {
                                return '';
                            }                    

                            $textmode = explode( ',', $params['text'] );

                            if ( $include->isLocked() )
                            {
                                $href = $include->getVar('resource');
                                $text = bpick( imgy($textmode[1]), 'Locked');
                            } 
                            else 
                            {
                                $href = $include->getVar('resource') . '!' . $params['view'];
                                $text = str_replace('{$title}', $include->getVar('title'), bpick( imgy($textmode[0]), 'Edit {$title}'));
                            }
                        break;
                        
                        case 'v':
                        case 'history':
                            $include->get();
                            
                            if ( $include->getVar('version') == 1 )
                            {
                                return '';
                            } 
                            else 
                            {
                                $href = $include->getVar('resource') . '!' . $params['view'];
                                $text = pick( imgy($params['text']), 'History' );
                            }
                        break;
                        
                        case null: //general link with view appended//
                            $href = $include->getVar('resource');
                            $text = pick(imgy(htmlspecialchars($params['text'])), $include->getVar('title'));                        
                        break;
                        
                        default: //simple link to resource with no modification//
                            $x = htmlspecialchars(preg_replace('/^[^\&]*/', '', $include->getVar('uri') ));
                            $href = $include->getVar('resource') . '!' . $params['view'] . $x;
                            $text = epick( imgy(htmlspecialchars($params['text'])), $params['view'] );                                    
                        break;
                    }
                    
                    return '<a href="'. $this->galmetapost->getVar('url_prefix') . $href . '">'. $text .'</a>';
                    
                break;          
            }

            $this->set('content',$content);
            $this->set('params',$params);
            $this->set('include',$include);
            $this->set('request',$this->galmetapost->getRequest());
            $this->set('prefix',$this->galmetapost->getVar('url_prefix'));

            $buffer = $this->renderTemplate( $template );        
        }
        return $buffer;
    }


    /*
     * Method: processEdit
     * Process a form submission containing resource content. This is installed as a hook 
     * at <GalmetaPostRequest::postDispatch()>. It will check for access rights, sanitize
     * content and remove any plugin tokens that the user doesn't have access to write.
     * 
     * Parameters:
     *  action - string Submit button value taken from $_POST['do']. Must be 'Submit'.
     * 
     * Return:
     *  boolean - True on success, false on error.
     */
    function processEdit( $action )
    {
        if ( $action != 'Submit' )
        {
            return null;
        }

        //check for a captcha verification//
        if ( $_SESSION['captcha_scope'] == 'all' || ( $_SESSION['captcha_scope'] == 'guest' 
            && (empty($_SESSION['username']) || $_SESSION['username'] == 'guest')  
            && $_SESSION['security_code'] != $_POST['captcha'] ) )
        {
            $this->messages[] = '<div class="error">Invalid security code</div>';
            $request =& $this->galmetapost->getRequest();
            $request->setVar('view',$_SESSION['captcha_view']);
            return null;
        }
        else
        {
            unset( $_SESSION['captcha_scope'] );
            unset( $_SESSION['captcha_view'] );
        }

        foreach ( $_POST['name'] as $index => $name )
        {
            $method = '0';
                    
            $edit = new GalmetaPostResource( $this->galmetapost, $name );
            $edit->get();

            if ( empty ($_POST['content'][$index]) )
            {
                $method = 'Delete';
            }
            elseif ( !$edit->getVar('autoid') && $edit->getVar('exists') )
            {
                $method = 'Update';
            }
            else
            {
                $method = 'Create';
            }

            //check if the user has access to method//
            if ( $edit->getVar('can_' . strtolower($method)) )
            {
                //apply safehtml for XSS attacks//
                if ( $this->galmetapost->getVar('safehtml') && $this->galmetapost->getUserSetting('safehtml',true) )
                {
                    define('XML_HTMLSAX3', $this->galmetapost->getVar('path') . '/_lib/');
                    require $this->galmetapost->getVar('path') . '_lib/safehtml.php';
                    $safehtml =& new safehtml();
                    $content = $safehtml->parse(deslash($_POST['content'][$index]));            
                } 
                else 
                {
                    $content = deslash($_POST['content'][$index]);            
                }

                $user = (empty($_SESSION['username'])?'guest':$_SESSION['username']);

                //restrict removing {include:} from core/
                if ( $edit->getVar('resource') == 'core' && !preg_match('/\{include\:\}/',$content) )
                {
                    return false;
                }

                //apply plugin write restrictions//
                $content = preg_replace( '/\{([^\}\s]+)\:(\s([^\}]+))?\}/e', '($this->galmetapost->checkAccess(\'{$1:}\',\'cud\')?\'$0\':\'\');', $content );

                switch ( $method ){
                    case 'Create':
                        //look for a recently deleted version and resurrect it//
                        if ( $this->galmetapost->getVar('versioning') && !$edit->getVar('autoid') )
                        {
                            if ( $list = $this->galmetapost->call( 'findAllByName', $edit->getVar('resource'), 'version', 'DESC', true ) )
                            {
                                $this->galmetapost->call( 'copyResource', $list[0]['name'], $edit->getVar('resource') );
                                $this->galmetapost->call( 'updateResource', $edit->getVar('resource'), $content, $user  );
                                return true;
                            }
                        }

                        if ( $edit->getVar('autoid') )
                        {
                            $edit->setVar('resource', str_replace('~',$this->galmetapost->getAutoId(),$edit->getVar('resource')));
                            $r =& $this->galmetapost->getRequest();
                            $this->galmetapost->redirect($edit->getVar('resource'));
                            //$r->setVar('resource',$edit->getVar('resource'));
                        }

                        $this->galmetapost->call( 'insertResource', $edit->getVar('resource'), $content, $user  );
                    break;

                    case 'Update':
                        $this->galmetapost->call( 'saveVersion', $edit->getVar('resource') );
                        $this->galmetapost->call( 'updateResource', $edit->getVar('resource'), $content, $user );
                    break;

                    case 'Delete':
                        $this->galmetapost->call( 'saveVersion', $edit->getVar('resource') );
                        $this->galmetapost->call( 'deleteResource', $edit->getVar('resource') );
                    break;

                    default:
                        //do nothing//
                        return false;
                    break;
                }

                return true;                
            }
        }

        return false;
    }    
    
    /*
     * Method: editorPlugins
     * Return a list of editor (fckeditor) plugin locations to include when rendering
     * editor. Meant to be used via hook system.
     * 
     * Parameters:
     * 	plugins - Array, plugins to pass-thru and concat too.
     * 
     * Return:
     * 	Array - array of system paths to php files to include.
     */
    function editorPlugins( $plugins = null )
    {
        $this->galmetapost->HTMLInclude('_lib/mootools.js');
    	$plugins = pick( $plugins, array() );
   
        return $plugins;
    }
}
?>