<?
/*
 * Class: GalmetaPostPlugin_login
 * Plugin which displays and processes a login request. Checks the registry keys
 * under "users/" and compares a SHA1 hashed password saved as the value for authentication.
 * This plugin will lock out the user after a number of login failures.
 *
 * Plugin Parameters:
 *  display - Tells the login plugin what to display, values include [form|link], defaults to form.
 * 
 * Update Log
 *  1.1 - added success attribute to forward successul login to another page.
 */
class GalmetaPostPlugin_login extends AbstractGalmetaPostPlugin {

    function GalmetaPostPlugin_login( &$G )
    {
        $this->name = 'Login';
        $this->version = '1.1';
        $this->author = 'Bryan English <bryan@galmeta.com>';
        $this->link = 'http://galmeta.com/post/';
        $this->copyright = 'Copyright &copy; 2006 Galmeta LLC';
        
        $this->usage = array(
            'display' => array(
              'default' => 'form',
              'description' => 'Tells the login plugin what to display, values include [form|link], defaults to form.'),

            'success' => array(
              'default' => null,
              'description' => 'URL, page to redirect to on successful login.')
            );
        $this->messages = array();

        parent::AbstractGalmetaPostPlugin( $G );
    }

    /* 
     * Method: install
     * Install a function to process a login request passed through postDispatch
     */
    function install()
    {
        $this->registerPluginHook('processLogin','GalmetaPostRequest/postDispatch');
        parent::install();
    }

    function update()
    {
        $this->install();
    }

    /*
     * Method: run
     * Run the plugin.
     *
     * Parameters:
     *  params - Plugin parameters passed from galmetapost resource or in-code call. See
     *  <usage> for more information on allowed parameters.
     *  resource - Resource context of this executing plugin.
     * 
     * Return:
     *  String - rendered content
     */
    function run( $params, &$resource )
    {
        $params = $this->getParameters( $params );
        
        if ( $params['display'] == 'link' )
        {
            if ( empty ( $_SESSION['username'] ) )
            {
                $this->set('href',$this->galmetapost->getVar('url_prefix') . 'login');
                $this->set('text','Login');
            } 
            else 
            {
                $this->set('href',$this->galmetapost->getVar('url_prefix') . 'logout');
                $this->set('text','Logout');                
            }
            
            return $this->renderTemplate('link');            
        }
        else
        {        
            $this->set( 'loggedIn', !empty($_SESSION['username']) );
            $this->set( 'failures', intval($_SESSION['failures']) );
            $this->set( 'messages', implode('<br />',$this->messages ) );
            $this->set( 'success', $params['success'] );

            return $this->renderTemplate();
        }
    }

    /*
     * Method: processLogin
     * This processes a login attempt if $action == 'Login'. It will check the $_POST['username']
     * and $_POST['password'] against the "users/" registry key for a match. If no match is found,
     * nothing happends and an error is issued to the login screen. If a match is found, a session
     * is started for that user and their access_control tokens are loaded.
     * 
     * Parameters:
     *  action - String, must equal "Login"
     */
    function processLogin( $action )
    {       
        if ( $action == 'Login' && !empty($_POST['username']) && !empty($_POST['password']) )
        {
            if ( $_SESSION['failures'] < $this->galmetapost->getVar('login_failures') )
            {
                $pass = $this->galmetapost->registry('users/' . $_POST['username']);

                if ( !empty($pass) ){

                    if ( $pass == SHA1($_POST['password']) )
                    {
                        //session_regenerate_id(); need to figure out why this doesn't work in an object
                        unset($_SESSION['tries']);
                        $_SESSION['username'] = $_POST['username'];
                        
                        if ( $_POST['success'] )
                        {
                            $this->galmetapost->redirect( $_POST['success'] );
                        }
                        else
                        {
                            $this->messages[] = '<span class="success">You are currently logged in as ' . htmlspecialchars($_SESSION['username']) .'</a>';
                            return true;
                        }
                    } 
                    else 
                    {
                        //track failed attempts//
                        if ( !empty($_SESSION['failures']) )
                        {
                            $_SESSION['tries'] += 1;
                        } 
                        else 
                        {
                            $_SESSION['tries'] = 1;
                        }

                        $this->messages[] = '<span class="error">Password incorrect!</a>';
                        return false;
                    }
                }
                else
                {
                    $this->messages[] = '<span class="error">Username does not exist!</a>';
                    return false;
                }
            }
            else
            {
                $this->messages[] = '<span class="error">Too many failed login attempts. You must restart your browser before attempting to login again.!</a>';
                return false;
            }
        }
    }
}

?>