<?
/**
 * Class: GalmetaPostPlugin_ls
 * Queries a list of resources found in a provided base-path using any number of additional
 * optional parameters to expand or limit the query.
 *
 * Plugin Parameters:
 *  path - String, Absolute or relative GalmetaPost path to the resource to list.
 *  xpath - String, Absolute or relative GalmetaPost wildcard-capable path to query.
 *  as - String, How to display the list. [link|summary|full|tree|menu] 
 *  sort - String, What field to sort on. 
 *  order - String, Direction to sort.
 *  recursive - Boolean, Controls if the list will include subdirectories. True if listing as menu
 *  pagination - Boolean, Controls if excess resources are pagenated.
 *  versions - Boolean, Flag if to list resource versions.
 *  offset - Integer, offset
 *  depth - Integer, If recursive is set this controls the depth
 *  silent - Depreciated in favor of on_empty. Boolean, If no resources found don't print anything. 
 *  style - String, Style of listing. [basic|full|tree] Used mainly for menu list. Defaults to 'basic'.
 *  limit - Integer, Limits the list.
 *  on_empty - String, optional, what to display when the list is empty. 
 *  core_resources - Boolean, Includes core resources (core,css,page_not_found,access_denied) in the listing.
 * 
 * Updates:
 *  1.1 - added file listing feature when path contains the GalmetaPostRequest::files_path.
 */

class GalmetaPostPlugin_ls extends AbstractGalmetaPostPlugin {

    function  GalmetaPostPlugin_ls( &$G ){
        $this->name = 'List';
        $this->version = '1.1';
        $this->description = 'Creates a list from GalmetaPost resources.';
        $this->author = 'Bryan English <bryan@galmeta.com>';
        $this->link = 'http://galmeta.com/post/';
        $this->copyright = 'Copyright &copy; 2007 Galmeta LLC';
        
        $this->usage = array(
            'path' => array(
                'default' => null,
                'description' => 'Absolute GalmetaPost path to the resource to list.'),

            'xpath' => array(
                'default' => null,
                'description' => 'eXtended path specifier similar to xpath to aggregate resources.'),
            
            'as' => array(
                'default' => 'link',
                'description' => 'How to display the list. [link|summary|full|tree|menu] '),
            
            'sort' => array(
                'default' => 'name',
                'description' => 'What field to sort on.'),
            
            'order' => array(
                'default' => 'ASC',
                'description' => 'Direction to sort.'),
            
            'recursive' => array(
                'default' => false,
                'description' => 'Controls if the list will include subdirectories. True if listing as menu'),
            
            'pagination' => array(
                'default' => false,
                'description' => 'Controls if excess resources are pagenated.'),
            
            'versions' => array(
                'default' => false,
                'description' => 'Flag if to list resource versions.'),
            
            'offset' => array(
                'default' => null,
                'description' => 'Offset'),
            
            'depth' => array(
                'default' => 3,
                'description' => 'If recursive is set this controls the depth'),
            
            'silent' => array(
                'default' => false,
                'description' => "If no resources found don't print anything."),
            
            'style' => array(
                'default' => 'basic',
                'description' => 'Style of listing. Used mainly for menu list.'),
            
            'limit' => array(
                'default' => null,
                'description' => 'Limits the list.'),

            'on_empty' => array(
                'default' => 'No Resources',
                'description' => 'What to output when the list is empty.'),
            
            'core_resources' => array(
                'default' => false,
                'description' => 'Includes core resources (core,css,page_not_found,access_denied) in the listing.')
        );
          
        parent::AbstractGalmetaPostPlugin($G);
    }

    /*
     * Method: run
     * Run the include plugin. Getting content to either read or edit.
     *
     * Parameters:
     *  params - Plugin parameters passed from galmetapost resource or in-code call. See
     *  <usage> for more information on allowed parameters.
     *  resource - Resource context of this executing plugin.
     * 
     * Return:
     *  String - rendered content
     */
    function run( $params, $context ) {
   
        $params = $this->getParameters( $params );        

        if ( $params['as'] == 'index' || $params['as'] == 'versions' || $params['pagination'] == true )
        {
            $params['pagination'] = true;
            $params['sort'] = ($_GET['sort']?slash($_GET['sort']):$params['sort']);
            $params['order'] = ($_GET['order']?slash($_GET['order']):$params['order']);
            $params['offset'] = intval($_GET['offset']?slash($_GET['offset']):$params['offset']);       
        }

        //if xpath is set
        if ( !empty($params['xpath']) )
        {
            $request =& $this->galmetapost->getRequest();
            
            //expand path
            $params['xpath'] = expand_path( $params['xpath'], $context->getVar('resource'), (($request)?$request->getVar('resource'):null));

            //findAllByName
            $params['xpath'] = str_replace('*','%',$params['xpath']);
            $total = $this->galmetapost->call( 'getCount', $params['xpath'], $params['versions'], true, $params['core_resources'] );
            $list = $this->galmetapost->call( 'findAllByName', $params['xpath'], $params['sort'], $params['order'], $params['versions'], $params['offset'], tpick($params['limit'],999999999), true, $params['core_resources']);
            $params['path'] = $context->getVar('resource');
        }
        //else check for path
        else
        {
            //if path is blank, use context//
            if ( empty($params['path']) )
            {
                $params['path'] = '/' . $context->getVar('resource') . '/';
            }

            //special case for all list//
            if ( $params['path'] == '/' )
            {
                $total = $this->galmetapost->call( 'getCount', '', $params['versions'], true, $params['core_resources'] );
                $list = $this->galmetapost->call( 'findAllByName','', $params['sort'], $params['order'], $params['versions'], $params['offset'], tpick( $params['limit'], 999999999 ), true, $params['core_resources']);
                $params['path'] = $context->getVar('resource');                
            }
            elseif ( preg_match('/^_files\/.*$/', $params['path'] ) && strpos($params['path'],'..') === false)
            {
                $p = preg_replace( '/^_files\//',$this->galmetapost->getVar('files_path'), $params['path'] );
                $files = glob($p);
                $total = count($files);
                $list = array();
                foreach ($files as $file)
                {
                    preg_match('/([^\/]+)\.([a-zA-Z0-9]+)$/',$file,$matches);
                    $list[] = array( 
                        'name' => $file , 
                        'content' => str_replace( '_', ' ', $matches[1] ), 
                        'ext' => $matches[2],
                        'edited_on' => date( $this->galmetapost->getVar('date_format'), filemtime($file)) );
                    $params['path'] = $context->getVar('resource');
                    $params['as'] = ( $params['as'] == 'link' ? 'files' : $params['as'] );
                }
            }
            else
            {
                $ls = new GalmetaPostResource( $this->galmetapost, $params['path'], $context->getVar('resource') );
                $ls->setVar('view', (($params['versions'])?'v': $ls->getVar('view')) );           
                $total = $ls->call('getCount');
                $list = $ls->call('getList', $params['sort'], $params['order'], $params['offset'], tpick( $params['limit'], 999999999 ), true, $params['core_resources'] );
            }


            //strip resources if not recursive//
            if ( !empty($list) && $params['as'] != 'menu' && $params['recursive'] == false 
                 && !$params['versions'] && isset($ls) && $ls->getVar('view') != 'v' ){
        
                $nlist = array();
                $unique = array();
                $regex_p = str_replace('/','\\/',$ls->getVar('resource') . '/');
        
                foreach ( $list as $page )
                {
                    $pos = false;
                    $matches = array();
                    
                    preg_match ( '/' . $regex_p . '([a-z0-9\-\_]+)(\/[a-z0-9\-\_])?/i', $page['name'], $matches );  
                    $matcher = $ls->getVar('resource') . '/' . $matches[1];
        
                    if ( !in_array( $matcher, $unique ) )
                    {
                        if ( $matches[2] == '' )
                        {
                            $nlist[] = $page;
                        } 
                        else 
                        {
                            $nlist[] = array( 'name' => $matcher);
                        }
                        $unique[] = $matcher;
                    }
                }
                $list = $nlist;
            }                                  
        }

        //look for empty list//
        if ( !$list )
        {
            if ( $params['silent'] == 'true' )
            {
                return '';
            } 
            else 
            {
                return $params['on_empty'];
            }
        }

        //set template//
        $this->set( 'prefix', $this->galmetapost->getVar('url_prefix') );
        $this->set( 'list', $list );
        $this->set( 'params', $params );
        $this->set( 'limit', tpick( $params['limit'], 30 ) );
        $this->set( 'total', $total );
    
        //special case for menu display, some recursive building//   
        if ( $params['as'] == 'menu' )
        {    
            $buffer = $this->makeMenu( $list, $params );
        } 
        else 
        {    
            $buffer = $this->renderTemplate( $params['as'] );
        }
        
        return $buffer;
    }    

    /*
     * Method: makeMenu
     * Creates an html from a list array and plugin parameters  
     * 
     * Parameters:
     *  list - Array, list of post resources queried from the database
     * 
     * Return:
     *  String - finished html menu
     */
    function makeMenu ( $list, $params )
    {
        $items = array();
        $buffer = '';
        if ( count($list) > 0 )
        {
            foreach ( $list as $page )
            {
                $items = $this->buildArray( $page['name'] , $items );
            }    
            $request =& $this->galmetapost->getRequest();
            $buffer = $this->makeMenuItems( $items, $params['style'], '', 0, $params['depth'], $request->getVar('resource') );
        }
        
        return $buffer;
    }   

    /*
     * Method: buildArray
     * Takes a resource path, breaks it apart at folders, rearranges folder and packs into a pre-existing
     * hierarchial array (list).  
     * 
     * Example:
     *  A path of "products/furniture/oak_chair" will result in array(
     *      'products' => array(
     *          'furniture' => array(
     *              'oak_chair' => true;
     *          )
     *      )
     *  ) 
     * 
     * Parameters:
     *  path - String, path of the resource to pack into a hierachial menu
     *  list - Array, nested array to pack the resourses into, taking care of duplicates and hierarchy.
     * 
     * Return:
     *  Array - nested array representing the breakdown of a resource path.
     */
    function buildArray( $path, $list )
    {
        $parts = explode('/',$path);
        $max = count($parts);
        $cur = &$list;

        for ( $cnt=0; $cnt<$max; $cnt++ )
        {   
            if ( $cur[$parts[$cnt]] && is_array($cur[$parts[$cnt]]))
            {
                $cur = &$cur[$parts[$cnt]];
            } 
            else 
            {
                if ( ($max-1) == $cnt )
                {
                    $cur[$parts[$cnt]] = true;            
                } 
                else 
                {
                    $cur[$parts[$cnt]] = array();
                    $cur = &$cur[$parts[$cnt]];
                }
            }
        }
        return $list;
    }

    /*
     * Method: makeMenuItems
     * Recursive method which constructs the actual HTML for a nested, menu. This method utilizes 2 templates
     * to build a menu, "basic.thtml" and "basic_items.thtml" when style is set to "basic". "basic.thtml" is used
     * at a content folder level, and each resource within a folder is rendered with "basic_items.thtml".
     * 
     * Parameters:
     *  tree - Array, nested array to recursivly parse and serve as the menu items.
     *  type - String, controls the templates (thtml) used to render the menu and menu items. Defaults to 'basic'.
     *      taken from the 'style' plugin parameter.
     *  link - String, recursively built string which creates the href's.
     *  depth - Integer, recursive parameter which keeps track how far the process in depth-wise.
     *  max_depth - Integer, optional, how deep to go when building the menu. defaults to 3. 
     *  current - String, path representing the current location of the user so menu opens correctly.
     * 
     * Return:
     *  string - HTML String of the complete menu. 
     */
    function makeMenuItems( $tree, $type, $link='', $depth=0, $max_depth=3, $current )
    {        
        $buffer = '';
        
        foreach ( $tree as $key => $value )
        {
            $items = '';
            $alink = $link . $key . '/';
            if ( preg_match( '/^'.$key.'(\/(.*)|$)/', $current, $matches ) )
            {
                $acurrent = $matches[2];
                $isCurrent = true;
            } 
            else 
            {
                $acurrent = $current;
                $isCurrent = false;
            }

            if ( is_array($value) && $depth < $max_depth )
            {
                $items = $this->makeMenuItems( $value, $type, $alink, ($depth+1), $max_depth, $acurrent );
            }

            $buffer .= $this->renderTemplate( array( 'url_prefix' => $this->galmetapost->getVar('url_prefix'), 
                'isCurrent' => $isCurrent, 
                'link' => $alink, 
                'items' => $items, 
                'item' => prettytext($key) ), $type . '_items' );
        }

        return $this->renderTemplate( array( 'items' => $buffer ) , $type );

    }
}
?>