<?php
/*
 * Class: GalmetaPostPlugin_moderate
 * This plugin allows a single point in the site to be monitored for 
 * moderation. Commonly used for user submitted comments. It will store
 * the submissions in a place you choose, peferrably under an access protected
 * page, and lets you approve, reject or modify them.
 * 
 * Plugin Parameters:
 *  watch - String, path to page to watch for submissions
 *  queue - String, path to publicly protected path to queue up content.
 *  email - String, optional, email to send notifications too.
 */
class GalmetaPostPlugin_moderate extends AbstractGalmetaPostPlugin 
{

    function GalmetaPostPlugin_moderate( &$n )
    {
        $this->name = 'moderate';
        $this->version = '1.0';
        $this->description = "Moderation plugin to monitor and manually approve comments and other branches of editable pages.";
        $this->author = 'Bryan English';
        $this->link = 'http://galmeta.com/post';
        $this->copyright = 'Galmeta LLC 2007';
        $this->usage = array(
            'watch' => array(
                'default' => 'comments',
                'description' => 'string, page path to watch for new content submissions.' 
            ),
            'queue' => array(
                'default' => 'editor/queue',
                'description' => 'string, publicly protected path to queue up content requiring approval.' 
            ),
            'email' => array(
                'default' => null,
                'description' => 'string, an email address to send an email to when a page needs approval.' 
            )
        );
        $this->messages = array();
                
        parent::AbstractGalmetaPostPlugin( $n );
    }
    
    function install()
    {
        //install css//
        $this->registerPluginHook('insertResource','GalmetaPostRequest/insertResource','before');        
        $this->registerPluginHook('approvalTab','include/editorPlugins','after');        
        parent::install();
    }

    function update()
    {
        $this->install();
    }

    /*
     * Method: run
     * Run the include plugin. Default method called by resource parser when found in
     * content.
     *
     * Parameters:
     *  params - Optional, Plugin parameters passed from galmetapost resource or in-code call. 
     *  resource - Resource context of this executing plugin. A reference to the calling resource.
     * 
     * Return:
     *  String - rendered content
     */
    function run( $params, &$resource )
    {
        $params = $this->getParameters( $params );

        $sane = $this->checkModeratedPageAccess($params['watch'],$params['queue']);

        if ( $sane ) //update moderation watchpoints//
        {            
            $sane['email'] = $params['email'];
            $old = $this->registry('watchpoints//');
            
            if ( $old )
            {
                $oldwatch = unserialize($old[0]['content']);
                $this->galmetapost->deleteResource($oldwatch['watch'] . '/moderate');            
                $this->registry('watchpoints//','');
            }            
            
            $this->registry('watchpoints/' . $sane['watch'], serialize($sane));
            $this->galmetapost->insertResource( $sane['watch'].'/moderate', 'Your submission has been marked for moderation by the site administrator.', $_SESSION['username']);
        }
        
        $watchpoints = $this->registry('watchpoints//');
        
        if ( $watchpoints === false )
        {
            return "No watch point set";
        }
        
        $mod = unserialize( $watchpoints[0]['content'] );
        
        return "Watching: {$mod[watch]}<br />Queue: {$mod[queue]}";
    }

    /*
     * Method: insertResource
     * Hook method added before GalmetaPostRequest::insertResource
     */
    function insertResource( $name, $content )
    {
        $check = $this->registry('watchpoints//');
        
        if ( $check !== false )
        {
            $mod = unserialize( $check[0]['content'] );

            if ( preg_match('/^'. preg_quote($mod['watch'],'/').'\/.+/', $name ) )
            {
                if ( $_POST['moderate'] && $this->galmetapost->checkAccess($name,'crud'))
                {
                    if ( $_POST['moderate'] == 'approved')
                    {
                        //update the page while in queue, the move to watchpoint//
                        $this->galmetapost->db->Execute("UPDATE `". $this->galmetapost->getVar('table_name') ."` SET content="
                            . $this->galmetapost->db->qstr($content) ." WHERE name=" 
                            . $this->galmetapost->db->qstr($mod['queue'] .'/'. $name)) or die(__LINE__.' '.$this->galmetapost->db->ErrorMsg());
                                               
                        $this->galmetapost->moveResource($mod['queue'] .'/'. $name, $name);
                        $this->galmetapost->redirect( $mod['queue'] . '/' );
                    }
                    else
                    {
                        //update the page while in queue, the move to watchpoint//
                        $this->galmetapost->deleteResource($mod['queue'] .'/'. $name);
                        $this->galmetapost->redirect( $mod['queue'] . '/' );                        
                    }                        
                }  
                else
                {
                    //swap the page with the moderation page//                
                    $notify = new GalmetaPostResource( $this->galmetapost, $mod['watch'].'/moderate' );                
                    $this->galmetapost->setRequest( $notify );
                    
                    return new HookPassCondition($mod['queue'] .'/'. $name,$content);
                } 
            }
        }
    }
         
    function approvalTab( $plugins = null )
    {
        $plugins = pick( $plugins, array() );
        $plugins[] = $this->galmetapost->getVar('path') . '_ext/plugins/moderate/editor.approval_tab.php';
        return $plugins;
    }
    
    function checkModeratedPageAccess( $watch, $queue )
    {
        //check permissions//
        $sane_watch = parse_uri( $watch );
        $sane_queue = parse_uri( $queue );
        
        $sane_watch = $sane_watch['resource'] . '/check';
        $sane_queue = $sane_queue['resource'] . '/check';
        
        if ( !$this->galmetapost->checkaccess($sane_watch,'crud') )
        {
            $this->messages[] = '<span class="error">Your user does not have sufficent rights to <strong>'. $sane_watch .'</strong></span>';
            return false;
        }
        
        if ( !$this->galmetapost->checkaccess($sane_queue,'crud') )
        {
            $this->messages[] = '<span class="error">Your user does not have sufficent rights to <strong>'. $sane_queue .'</strong></span>';
            return false;
        }
        
        return array( 'watch' => $sane_watch, 'queue' => $sane_queue );
     }       
}     
?>