<?php

/*
 * Class: GalmetaPostPlugin_users
 * This plugin manages user accounts, passwords and settings. Please note 
 * that this doesn't do access control. All information is stored in the 
 * registry under the 'users/' key.
 * 
 * See Also:
 *  <GalmetaPostPlugin_access>
*/
class GalmetaPostPlugin_users extends AbstractGalmetaPostPlugin {

    function GalmetaPostPlugin_users( &$n )
    {
        $this->name = 'Users';
        $this->version = '1.0';
        $this->description = 'Admin plugin which creates users and sets passwords. This plugin allows provides an interface to add user-level settings.';
        $this->author = 'Bryan English <bryan@galmeta.com>';
        $this->link = 'http://galmeta.com/post/';
        $this->copyright = 'Copyright &copy; 2007 Galmeta LLC';
        $this->messages = array();

        parent::AbstractGalmetaPostPlugin( $n );
    }

    function install()
    {
        $this->registerPluginHook('getUserSetting','GalmetaPostRequest/getUserSetting');
        $this->registerPluginHook('updateUsers','GalmetaPostRequest/postDispatch');
        parent::install();
    }

    function update()
    {
        $this->install();
    }

    /*
        Method: run
        Displays the user management interface

        Return:
            string - HTML content of interface
    */
    function run()
    {
        $content = '';        

        if ( $this->galmetapost->request->resource == 'admin/users' && $this->canAccess() )
        {            
            $this->galmetapost->HTMLInclude('_lib/mootools.js');
            $this->galmetapost->HTMLInclude('_lib/prototype.js');

            $users = $this->getUsers();

            if ( count($users) > 0 )
            {        
                $this->set('messages', implode('<br />',$this->messages) );

                foreach ( $users as $user ) 
                {
                    $content .= $this->renderTemplate(array('username' => $user, 'settings'=> $this->getUserSettings( $user ) ),'user');
                }

                $this->set('users',$content);
                $this->set('urlPrefix',$this->galmetapost->getVar('url_prefix'));
            }    
        }

        return $this->renderTemplate('users');
    }

    /*
        Method: updateUsers
        Processes posted user update data. Handles user creation, modification and deletion and adding of user 
        specific settings. Hooks into the postDispatch function.

        See Also
            <GalmetaPostRequest::postDispatch()>
    */
    function updateUsers( $action )
    {
        $request =& $this->galmetapost->getRequest();
        
        //process submission//
        if ( $request->getVar('resource') == 'admin/users' && $action == 'Update Users' && $this->canAccess() )
        {                
            $deletions = array();

            foreach ( $_POST as $name => $value )
            {
                //filter out all non user post fields//
                if ( !empty($value) && preg_match('/^(new\/)?users\/([a-zA-Z0-9_]+)$/',$name,$matches) )
                {
                    if ( in_array( $matches[2], $deletions ) )
                    {
                        continue;
                    }

                    //if new user fields//
                    if ( $matches[1] == 'new/' )
                    {        
                        if ( $this->_validate_password( $_POST['new/users/password'], $_POST['new/users/password_verify'] ) 
                        && $this->_validate_username( $_POST['new/users/username'] ) )
                        {
                            $this->galmetapost->registry('users/' . $_POST['new/users/username'], SHA1($_POST['new/users/password']) );
                            $this->messages[] = '<div class="error">New user successfully created.</div>';
                        }

                    //modify existing user//
                    } 
                    else
                    {
                        if ( $this->_validate_password( $_POST[$name], $_POST[$name . '/verify'] ) )
                        {
                            $this->galmetapost->registry($name, SHA1($value) );
                            $this->messages[] =  '<div class="error">User modified successfully.</div>';
                        }
                    }
                } 
                //check for user settings updates//
                elseif ( preg_match('/^(new\/)?users\/([a-zA-Z0-9_]+)\/settings\/([a-zA-Z0-9_]+)$/',$name,$matches) )
                {
                    if ( in_array( $matches[2], $deletions ) )
                    {
                        continue;
                    }

                    if ( $matches[1] == 'new/' )
                    {
                        if ( $matches[3] == 'name' )
                        {
                            foreach ( $value as $sid => $sname )
                            {
                                if ( preg_match('/^[a-zA-Z0-9_]+$/',$sname) )
                                {
                                    $this->galmetapost->registry( "users/{$matches[2]}/settings/{$sname}", $_POST["new/users/{$matches[2]}/settings/value"][$sid] );
                                }
                            }
                        }
                    }
                    else
                    {
                        $this->galmetapost->registry( $name, $value );
                    }
                }
                //check for user deletion//
                elseif ( preg_match( '/^delete\/([a-zA-Z0-9_]+)$/', $name, $matches ) )
                {
                    //delete user and settings//
                    $this->galmetapost->registry('users/' . $matches[1] . '//', '' );
                    $this->galmetapost->registry('users/' . $matches[1], '' );
                    $this->messages[] = '<div class="error">User '. $matches[1] .' deleted successfully.</div>';
                    $deletions[] = $matches[1];
                }
            }
        }
    }

    /*
        Method: _validate_password
        Validates for a somewhat good password length

        Parameters:
            username - String, password to validate.
            verify - String, verification password.
            
        Return:
            boolean - true if valid, false otherwise.

     */
    function _validate_password ( $password, $verify ) {
        if ( $password != $verify )
        {      
            $this->messages[] = '<div class="error">User passwords do not match.</div>';
        }

        if ( !preg_match( '/^.{8}/', $password ) )
        {
            $this->messages[] = '<div class="error">Passwords must be 8 characters long.</div>';
        }

        return ( (count($this->messages) == 0) );
    }

    /*
        Method: _validate_username
        validates a username, checks for non-alphanumerics and duplicate usernames.
        
        Parameters:
            username - String, username to validate.
            
        Return:
            boolean - true if valid, false otherwise.
    */
    function _validate_username ( $username )
    {
        if ( !preg_match('/^[a-z0-9_]+$/i',$username) )
        {
            $this->messages[] = '<div class="error">Usernames can only contain letters, numbers or underscores.<div>';
        }
        
        if ( in_array($username, $this->getUsers()) )
        {
            $this->messages[] = '<div class="error">Username already exists.</div>';
        }

        return ( (count($this->messages) == 0) );
    }

    /*
     * Method: getUsers
     * Gets and returns a list of usernames registered as users in post.
     * 
     * Return:
     *  Array - Usernames found in registry  
     */
    function getUsers()
    {
        $users = $this->galmetapost->registry('users/');
        $usernames = array();

        foreach ( $users as $user )
        {
            $usernames[] = preg_replace('/^users\//','',$user['name']);
        }

        return $usernames;
    }

    /*
        Method: getUserSettings
        Get's all of a users's settings from the registry

        Parameters:
            user - Username of the setting to get

        Return:
            mixed - array of settings
    */
    function getUserSettings( $user )
    {
        $settings = $this->galmetapost->registry('users/' . $user . '/settings/');

        if ( $settings )
        {
            foreach ( $settings as $id => $setting )
            {
                $settings[$id]['name'] = preg_replace('/^users\/' . $user . '\/settings\//','',$setting['name']);
            }
        }

        return $settings;
    }

    /*
        Method: getUserSetting
        Get's a users's setting from the registry. These are settings set by
        the administrator and not visible by the users them selves. It includes
        various user specific limitations.
 
        Parameters:
            setting - Name of the setting
            default  - Optional, value to return if setting is undefined.
            username - Optional, Username of the setting to get

        Return:
            mixed - null if no setting, otherwise value of the setting.

        See Also:
            <GalmetaPostRequest::getUserSetting()>
    */
    function getUserSetting( $setting, $default = null, $username = null  )
    {
        $username = ( $username == null ? $_SESSION['username'] : $username);

        if ( $setting = $this->galmetapost->registry('users/' . $username . '/settings/' . $setting) )
        {
            if ( is_numeric($setting) )
            {
                return intval($setting);
            } 
            elseif ( is_string($setting) && !preg_match("/true|false/",$setting))
            {
                return $setting;
            } 
            elseif ( is_string($setting) && preg_match("/true|false/",$setting))
            {
                return (($setting=='true')?true:false);
            }
            else
            {        
                return $setting;
            }
        }
        else
        {
            return $default;
        }
    }

}
?>