<?php
session_start();

/* Class: PIS
 * PIS (PHP Installation System) is an abstract class which when extended is made into a handy, web-based
 * installation package.
 * 
 * Note:
 *  THIS IS NOT MEANT TO BE USED BY ITSELF, RATHER AS A PARENT CLASS TO THE ACTUAL
 *  INSTALLATION IMPLEMENTATION.
 *
 * Version: 
 *  0.3
 * 
 * Author:
 *  Bryan English
 * 
 * Web site:
 *  http://bluelinecity.com/pis
 *
 * License:
 *  Copyright (c) 2006 Bryan English (bryan@bluelinecity.com)
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 *  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 *  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Diagram:
 * >    
 * >       [pis.php]      [MyInstaller.php]     [install.css]
 * >           |                 |                    |
 * >           +-----------------+--------------------+
 * >                             | 
 * >                        [install.php]
 * 
 * Todo:
 *  - Add Updating features to auto-update through a list of versions
 *  - Add custom input forms
 */
class PIS 
{

    /*
     * Property: buffer
     * String, this is the per-page running-buffer built up then displayed.
     */
    var $buffer;

    /*
     * Property: errors
     * Array, this will contain any field validation errors and will stop the user
     * from continuing until all validation tests have been met. 
     */
    var $errors = array();

    /*
     * Property: failures
     * Array, this will contain any <test()> failures which result in the next button
     * being disable and require the user to go back and fix something.
     */
    var $failures = array();

    /*
     * Property: pages
     * Array, names of the installation pages in the order they should be displayed. This
     * is essentially a list of the derived classes "page_*" methods.
     * 
     * Example:
     * > function page_welcome() ---> $pages[0] = 'welcome'
     * > function page_configs() ---> $pages[1] = 'configs'
     */
    var $pages = array();

    /*
     * Property: page_titles
     * Array, nicely formated names of the installation pages in the order they should be 
     * displayed. They will be auto-generated from the <pages> array however can be
     * overridden using the <title()> method.
     * 
     * See Also:
     *  <pages>
     *  <title()>
     * 
     * Example:
     * > function page_welcome() ---> $pages[0] = 'Welcome'
     * > function page_configs() ---> $pages[1] = 'Configs'
     * > function pageConnectionTest() ---> $pages[1] = 'Connection Test'
     */
    var $page_titles = array();

    /*
     * Property: title
     * String, title of your installation program if you choose to rebrand it. This is
     * passed to the constructor as the first parameter and is displayed on every installation
     * page. Can be thought of as a header too.
     * 
     * See Also:
     *  <footer>
     * 
     * Example:
     *  class MyInstaller extends PIS { ... }
     *  $installer = new MyInstaller("MyProgram's Super Installer");
     */
    var $title;

    /*
     * Property: current_page
     * Integer, index of the currently displayed page in reference to the <pages> array.
     */    
    var $current_page;
    
    /*
     * Property: footer
     * String, text to display as the installation footer on every page. Defaults to 
     * "PHP Installation System vX.X &copy; 2006 - bluelinecity.com/pis"
     * 
     * See Also:
     *  <title> 
     */
    var $footer;

    /*
     * Property: version
     * String, just a place to hold the version number of PIS.
     */   
    var $version;
    
    /*
     * Property: direction
     * Boolean, last direction the user has gone, true = forward (+1), false = backward (-1).
     * This is used to (inc/dec)riment <current_page>
     */   
    var $direction;

    /*
     * Method: PIS
     * Constructor for PIS, parses member methods looking for page definitions
     * and autocreates a step chain for them depending on their definition order.
     *
     * Parameters:
     *  title - String, <title> of the installation program.
     *  footer - Optional, <footer> to add. Defaults to "PHP Installation System ..."
     */
    function PIS($title='',$footer=NULL)
    {
        $this->title = $title;        
        $this->version = '0.3';

        if ($footer == NULL)
        {
            $this->footer = 'PHP Installation System v'.$this->version.' &copy; 2006 - bluelinecity.com/pis';
        }

        //look at self and all defined methods//
        $pages = get_class_methods(get_class($this));

        //auto create page chain from functions named "page*()"//
        foreach ( $pages as $page )
        {
            if ( preg_match('/^page[A-Za-z_]+/',$page) )
            {
                $this->chain(preg_replace('/^page_?/','',$page));
            }
        }

        //look for or create completion page//
        if ( !preg_match("/complete/i",$this->pages[count($this->pages)-1]) )
        {
            //if no pageComplete or page_complete defined, use my own.
            $this->chain('complete');
        }

        //clear the buffer//
        $this->clear();
    }

    /*
     *  Method: set
     *  Set an installation variable, basically a session variable. This is automatically
     *  called after each page submission thereby saving all form fields, (POSTs)
     *  during the entire installation process. Care should be given to treat each
     *  page's input field's ids as part of a giant form so vars don't get overwritten.
     *  Special variables denoted with 2 prefixed underscores (ie "__submit") are set 
     *  but not used by utility functions. 
     *
     *  Parameters:
     *      name - Name of the variable
     *      value - Value of the variable
     * 
     *  Return:
     *      mixed - Newly created session variable.
     */
    function set($name, $value )
    {      
        $_SESSION[$name] = $value;
        return $_SESSION[$name];
    }

    /*
     *  Method: get
     *  Gets an installation varible, basically a session variable. Special variables 
     *  denoted with 2 prefixed underscores (ie "__submit") can also be retrieved.
     * 
     *  Parameters:
     *      name - Name of the variable
     *      default - Value to return if requested variable is NULL
     * 
     *  Return:
     *      mixed - Value of the session variable
     */
    function get( $name, $default = null)
    {      
        if ( $_SESSION[$name] === null )
        {
            return $default;
        } 
        else 
        {
            return $_SESSION[$name];
        }
    }

    /*
     *  Method: areErrors
     *  Checks for any errors, encoutered by any <test> calls or required field
     *  validations on the current page.
     *
     *  Return:
     *      Boolean - Returns true if there were 1 or more errors, false otherwise.
     */
    function areErrors()
    {
        return (count($this->errors) > 0);
    }

    /*
     *  Method: title
     *  Overrides the default title of the current page.
     *
     *  Parameters:
     *      title - String, title to use instead of the auto-gened one in the constructor
     *         
     *  See Also: 
     *      <page_titles>
     *      <PIS()>
     */
    function title($title)
    {
		$this->page_titles[$this->current_page] = $title;
    }

    /*  
     * Method: message
     * Adds a message to the installation page's buffer. A simple line/block of text.
     * This uses an HTML <div /> element with the css class "message".
     * 
     * Parameters:
     *  text - The text to display.
     *  style - Optional, css style to add to it. Defaults to 'default'
     * 
     * Return:
     *  Boolean - true
     * 
     * Example:
     *  $this->message("Apache Module Confirmed!","confirm");
     *  -- creates this --
     *  <div class="message confirm">Apache Module Confirmed!</div>
     * 
     */
    function message($text,$style='default')
    {
        $this->buffer .= '<div class="message '. $style .'">'. $text .'</div>';
        return true;
    }

    /*
     *  Method: chain
     *  Appends a page and title to the <pages> and <page_titles> array.
     *
     *  Parameters:
     *      page - String, mame of the page (method) to append to the chain.
     *      title - Optional, Title of the page, will default to a formatted 
     *              version of $page parameter.
     * 
     *  Return:
     *      boolean - Returns true on success.
     */
    function chain( $page, $title = null)
    {
        if ( empty($title) )
        {
            $title = preg_replace('/([a-z])([A-Z])/','\1 \2',$page);
            $title = preg_replace('/([a-z])_([a-z])/','\1 \2',$title);
            $title = ucwords($title);
        }
    
        $this->pages[] = $page;
        $this->page_titles[] = $title;

        return true;
    }

    /*  
     *  Method: test
     *  A test to attempt before continuing to the next page. If the test variable is 
     *  true, it passes, else, it logs a message and performs an optional action.
     *
     *  Parameters:
     *      test - Boolean, a result from a test.
     *      message - Error message to log if one.
     *      action - Optional, page to jump to as a result of the error.
     * 
     *  Return:
     *      boolean - true on successful test, false otherwise. Can be used 
     *      to do conditional nesting in the installation. 
     */
    function test($test, $message, $action=null)
    {
        if ( !$test )
        {
            $this->failures[] = $message;

            if ( $action )
            {
                $this->clear();
                $this->page($action);
            }
        }

        return $test;
    }

    /*  
     *  Method: callPage
     *  Look for the named page in the derived class's method list and execute it. If it can't be
     *  found, show an error page. Used internally.
     *
     *  Parameters:
     *      name - Name of the method/page to call.    
     */
     function callPage($name)
     {
        if ( method_exists($this,'page' . $name) )
        {
            eval("\$this->page{$name}();");
        } 
        elseif ( method_exists($this,'page_' . $name) )
        {
            eval("\$this->page_{$name}();");
        } 
        elseif ( $name == 'complete' )
        {
            $this->completePage();
        } 
        else 
        {
            $this->errorPage("The page named {$name} could not be found.");
        }
    }

    /*  
     *  Method: page
     *  Call a page method as defined in the derived class and execute it, compiling
     *  any failures, messages, tests, etc into the class buffer. This puts a bulk of
     *  the page together into the <buffer> property.
     *
     *  Parameters:
     *      name - Name of the method/page to call.
     */
    function page($name){          

        $this->callPage($name);

        $failures = '';
        foreach ( $this->failures as $failure )
        {
            $failures .= '<div class="error">'. $failure .'</div>';
        }

        $this->buffer = '<h1>'. $this->title .'</h1><div id="page" class="'
            . $name .'"><h2>'. $this->page_titles[$this->current_page] 
            .'</h2><div id="main">' . $failures . $this->buffer . '</div>';

        $this->buffer .= '<div id="navigation">';

        //if no serious errors then show next button//
        if ( $failures == '' )
        {
            if ( $this->pages[$this->current_page+1] == 'complete' )
            {
                $this->input('complete');
            } 
            elseif ( $this->current_page < (count($this->pages)-1) ) 
            {
                $this->input('next');
            }
        }

        //if not first or completed, show back button//
        if ($this->current_page > 0 && $name != 'complete')
        {
            $this->input('back');
        }
        
        $this->buffer .= '</div><div style="clear:both"> </div></div>';

        //if not empty, print out footer//
        if (!empty($this->footer))
        {
            $this->buffer .= '<div id="footer">' . $this->footer . '</div>';
        }
    
        //if last page, destroy session vars//
        if ($name == 'complete')
        {
            $this->cleanup();
        }
    }

    /*
     * Method: errorPage
     * Called incase of a "page not found" error.
     *
     * Parameters:
     *  error - Error message to add to the page.
     */
    function errorPage($error)
    {
    	$this->title('Page Not Found');
    	$this->message($error);
    }

    /*
     * Method: completePage
     * Default Installation Complete page. If the page_complete() method is not 
     * defined in the derived class, this function is called instead.
     */
    function completePage(){
        $this->title('Installation Complete');
        $this->message('Please remove the installation file ( <strong>'. $_SERVER['SCRIPT_NAME'] .'</strong> ).');
    }

    /*
     * Method: makeConfig
     * Makes a configuration file from the session variables without a double 
     * underscore (__) prefix.
     * 
     * Parameters:
     *  template - Optional, String of php code to be eval'd and used on each 
     *      session var to generate the config file. Must contain $name and $value.
     *      Defaults to \${$name} = {$value}.
     *  header - Optional, config header text to add. Must be PHP code.
     *  saveTo - Optional, Path to save file too.
     *  exclusions - Optional, array of vars to leave out of the config file.
     * 
     * Return:
     *  mixed - string if no saveTo file provided, true if file saved okay, false if file write error.
     */
    function makeConfig($template="\\\${\$name} = {\$value};\n", $header='', $saveTo=null, $exclusions=null){
        
        if ( !$exclusions )
        {
        	$exclusions = array();
        }
        
        $config = "<?php\n/* Configuration file generated by a PHP Installation System (PIS) */\n\n";
		$config .= $header;

        foreach ( $_SESSION as $name => $value )
        {
            if ( !preg_match('/^__/',$name) && !in_array($name,$exclusions) && $name != session_name())
            {
                if ( is_numeric($value) )
                {
                    $value = intval($value);
                } 
                elseif ( is_string($value) && !preg_match("/true|false/",$value))
                {
                    $value = "'" . addslashes($value) . "'";
                } 
                elseif ( is_bool($value) )
                {
                    $value = (($value)?'true':'false');
                }

                eval("\$config .= \"{$template}\";");
            }
        }

        $config .= "\n?".'>';

        if ( $saveTo )
        {
            $f = fopen($saveTo,'w');
            if ( fwrite($f,$config) === false )
            {
                return false;
            }
            fclose($f);
            return true;
        }

        return $config;
    }

    /*
     * Method: clear
     * Clear the <buffer>
     *
     * Return:
     *  string - Old buffer
     */
    function clear()
    {
        $old = $this->buffer;
        $this->buffer = '';
        return $old;
    }

    /*
     * Method: hr
     * A simple horizontal rule added to the page buffer
     */
    function hr(){
        $this->buffer .= '<hr />';
    }

    /*
     * Method: cleanup
     * Destroy the session vars
     */
    function cleanup(){
        session_destroy();
    }

    /*
     * Method: randomKey
     * Generates a random string of characters for password generation.
     *
     * Parameters:
     *  length - Length of string to create.
     * 
     * Return:
     *  string - Random String.
     */
    function randomKey($length)
    {
        $pattern = "1234567890abcdefghijklmnopqrstuvwxyz";
        for($i=0;$i<$length;$i++)
        {
            if(isset($key))
            {
                $key .= $pattern{rand(0,35)};
            }
            else
            {
                $key = $pattern{rand(0,35)};
            }
        }
        return $key;
    }

    /*
     * Method: bypass
     * Create a bypass across any number of installation steps. Useful for
     * non-essential pages or for configuration jumps.
     *
     * Parameters:
     *  next - Page to goto when going "forward"
     *  back - Page to goto when going "back"
     */
    function bypass($back,$next)
    {
        $this->clear();
 
        $page = ($this->direction?$next:$back);
        
        $this->current_page = array_search($page,$this->pages);
        $this->callPage($page);
    }

    /*
     * Method: input
     * Factory method for adding HTML form inputa to the buffer for gathering installation configuration
     * information from the user. Any user-input capture by these form fields are automatically
     * added to the installer's SESSION using the fields id/name as the key.
     *
     * Parameters:
     *  type - Type of input to create, can be: 
     *      - textarea - creates a textarea input
     *      - select - creates a select box. Its options are specified in the "options" 
     *        attribute as an associative array. The key determines the label, value the option value.
     *      - boolean - creates a true/false dropdown for those config file flags.
     *      - agreement - creates an agreement input consisting of a textarea and 
     *        checkbox asking for agreement to the license. This input accepts an additional attribute 
     *        named file which when set to the path of the license file (mit.txt,gnu.txt,etc) will load 
     *        it into the textarea.
     *      - next - generally not used but by parent class
     *      - back - generally not used but by parent class
     *      - complete - generally not used but by parent class
     *      - text - creates a simple text input field.
     *      - password - creates a simple password input field.
     *      - hidden - creates a hidden form field.
     *  id - Optional, id of the input, used to identify throughout the installation process. And is
     *      what identifies the session variable, ie $this->get('name') will return this value at 
     *      any point of the installation
     *  extra - Optional, array of extra parameters, basically anything that goes into a form input.
     *      - label - The label for the field. Defaults to input id parameter.
     *      - value - initial value for the input
     *      - cols - Textarea columns
     *      - rows - Textarea rows
     *      - help - The help string for more information on the field. This string is pushed into a 
     *        javascript alert box and triggered by adding a "?" next to the field.
     *      - required - Will require the user to enter something.
     *      - options - Currently only for select input, must be an associative array of key=value pairs.
     *      - wrap - Textarea field for wrapping option [hard|soft|virtual]
     *      - size - For text fields.
     *      - file - Currently only for agreement input, path to a license text file to load into a page.
     *      - validation - Regular expression which must match the input value in order to pass validation
     *      - validation_message - What to display if validation regexp fails.
     * 
     *  Return:
     *      boolean - True on success
     * 
     * Todo:
     *  Move all this HTML out into a template.
     * 
     * See Also:
     *  <set()>
     *  <get()>
     *  <buffer>
     */
    function input( $type, $id=null, $extra=null ){


        $extra['value'] = $this->get($id,$extra['value']);

        switch ($type){
            case 'textarea':
                $this->buffer .= '<div class="field" id="div/'. $id .'">';
                $this->buffer .= '<label for="'.$id.'"'. ($extra['required']?' class="required">*':'>') 
                    . ($extra['label']?$extra['label']:$id).':</label> <textarea id="'. $id .'" name="'
                    . $id .'" cols="'. (isset($extra['cols'])?$extra['cols']:30) .'" rows="'
                    . (isset($extra['cols'])?$extra['rows']:7) .'" wrap="'
                    . (isset($extra['cols'])?$extra['wrap']:'soft') .'">'. $extra['value'] .'</textarea>';
            break;

            case 'select':
                $this->buffer .= '<div class="field" id="div/'. $id .'">';
                $this->buffer .= '<label for="'.$id.'"'. ($extra['required']?' class="required">*':'>') 
                    . ($extra['label']?$extra['label']:$id).':</label> <select name="'.$id.'" id="'.$id.'" >';
                foreach ( $extra['options'] as $key => $value){
                    $this->buffer .= '<option value="'.$value.'"'. ($value==$extra['value']?' selected="selected"':'') .'>'. (is_numeric($key)?$value:$key) .'</option>';
                }
                $this->buffer .= '</select>'; 
            break;

            case 'boolean':
                if ( $extra['value'] === true ){
                    $extra['value'] = 'true';
                }

                if ( $extra['value'] === false ){
                    $extra['value'] = 'false';
                }
                $this->buffer .= '<div class="field" id="div/'. $id .'">';
                $this->buffer .= '<label for="'.$id.'"'. ($extra['required']?' class="required">*':'>') . ($extra['label']?$extra['label']:$id).':</label> <select name="'.$id.'" id="'.$id.'" >';
                $options = array('false'=>'No','true'=>'Yes');
                foreach ( $options as $key => $value){
                    $this->buffer .= '<option value="'.$key.'"'. ($key==$extra['value']?' selected="selected"':'') .'>'.$value.'</option>';
                }
                $this->buffer .= '</select>'; 
            break;

            case 'agreement':
                if ( isset( $extra['file'] )){
                    $agreement = implode("",file($extra['file']));
                }
                $id = '__agreement';
                $this->buffer .= '<div class="field" id="div/'. $id .'"><pre>'. $agreement .'</pre>';
                $this->buffer .= '<input type="checkbox" '. (($extra['value']=='Y')?'checked="checked" ':'') .'value="Y" name="'.$id.'" id="'.$id.'" /><strong> I agree to the license terms above.</strong>'; 
            break;

            case 'next':
            case 'back':
            case 'complete':
                $this->buffer .= '<input type="submit" name="__submit" value="'. ucfirst($type) .'" />';
                return true;
            break;

            case 'checkbox':
                $this->buffer .= 'This input is not working yet. <div class="field" id="div/'. $id .'">';
                $this->buffer .= '<label for="'.$id.'"'. ($extra['required']?' class="required">*':'>') . 
                    (isset($extra['label'])?$extra['label']:$id).':</label> <input type="checkbox" '. 
                    ($extra['value'] == 'true'?'checked ':'') .
                    'value="Y" name="__checkbox_fix_'.$id.'" id="'.$id.'" onClick="setTimeout(\'document.forms[0].elements.'.$id.'.value = (document.forms[0].elements.__checkbox_fix_'.$id.'.checked?\\\'true\\\':\\\'false\\\'); alert(document.forms[0].elements.'.$id.'.value);\',100)" /> <input type="hidden" value="'. ($extra['value'] == 'true'?'true':'false') .'" name="'.$id.'" />'; 
            break;

            case 'text':
            case 'radio':
            case 'password':
            case 'hidden':
            case 'button':
                $this->buffer .= '<div class="field" id="div/'. $id .'">';
                $this->buffer .= '<label for="'.$id.'"'. ($extra['required']?' class="required">*':'>') . (isset($extra['label'])?$extra['label']:$id).':</label> <input type="'.$type.'" '. ($type=='checkbox' && !empty($extra['value'])?'checked ':'') .'value="'.(isset($extra['value'])?$extra['value']:'').'" name="'.$id.'" id="'.$id.'" size="'.(!empty($extra['size'])?$extra['size']:35).'" />'; 
            break;

            default:
                if ( method_exists($this,'input_' . $type) )
                {
                    eval("\$this->input_{$type}(\$id,\$extra);");
                }                
                return false;
            break;
        }

        if ( $extra['required'] )
        {
            $this->buffer .= '<input type="hidden" id="__required_'. $id .'" name="__required_'. $id .'" value="'. $id .'" />';
        }
                      
        if ( $extra['validation'] )
        {
            $this->buffer .= '<input type="hidden" id="__validation_'. $id .'" name="__validation_'. $id .'" value="'. htmlspecialchars($extra['validation']) .'" />';
        }

        if ( $extra['validation_message'] )
        {
            $this->buffer .= '<input type="hidden" id="__validation_message_'. $id .'" name="__validation_message_'. $id .'" value="'. htmlspecialchars($extra['validation_message']) .'" />';
        }
                      
        if ( !empty($extra['help']) )
        {
            $this->buffer .= '<div class="help"><a href="javascript:alert(\''. addslashes($extra['help']) .'\');">?</a></div> ';
        }

        if ( !empty($this->errors[$id]) )
        {
            $this->buffer .= '<div class="error">'. $this->errors[$id] .'</div> ';
        }

        $this->buffer .= '<div style="clear:both"> </div></div>';
        return true;
    }

    /*
     * Method: importSQLFile
     * Import an SQL File, this will open an sql dump file and execute it on the provided ADODB connection.
     *
     * Parameter:
     *  src - Path to the sql file to import
     *  conn - ADODB connection handle.
     *  patterns - Array or string of RegExp patterns to look for in the sql dump.
     *  replaces - Array or string of RegExp replacments to use with the patterns above.
     *  onerror - What to do on mysql error, nothing, warn, fail
     * 
     * Return:
     *  boolean - True on success, Array of errors on failure.
     * 
     * Todo:
     *  Merge this and <importMySQLFile> some how.
     */
    function importSQLFile($src, $conn, $patterns=null, $replaces=null, $onerror="nothing")
    {
        $file_content = file($src);
        $query = "";
        $errors = array();
        foreach ($file_content as $sql_line)
        {
            $tsl = trim($sql_line);
            if (($tsl != "") && (substr($tsl, 0, 2) != "--") && (substr($tsl, 0, 1) != "#"))
            {
                $query .= $sql_line;
                if(preg_match('/;\s*$/', $sql_line))
                {
                    if ( $patterns != null && $replaces != null)
                    {
                        $query = preg_replace($patterns,$replaces,$query);
                    }

                    if ( !$result = $conn->Execute($query) )
                    {
                        switch ($onerror)
                        {
                            case 'warn':
                                $errors[] = $conn->ErrorMsg();
                            break;
                            case 'fail':
                                die($conn->ErrorMsg());                                
                            break;
                            default:                            
                            break;
                        }
                    }
                    $query = "";               
                }
            }
        }
        
        if ( count($errors) > 0 )
        {
            return $errors;
        }
        else
        {
            return true;
        }
    }


    /*
     * Method: importMySQLFile
     * Import a MySQL File, this will open an mysql dump file and execute it on the provided mysql connection.
     *
     * Parameter:
     *  src - Path to the sql file to import
     *  conn - Mysql Database connection handle.
     *  patterns - Array or string of RegExp patterns to look for in the sql dump.
     *  replaces - Array or string of RegExp replacments to use with the patterns above.
     *  onerror - What to do on mysql error, nothing, warn, fail
     * 
     * Return:
     *  boolean - True on success, Array of errors on failure.
     * 
     * Todo:
     *  Merge this and <importSQLFile> some how.
     */
    function importMySQLFile( $src, $conn, $patterns=null, $replaces=null, $onerror='nothing')
    {
        $file_content = file($src);
        $query = "";
        $errors = array();
        foreach ($file_content as $sql_line)
        {
            $tsl = trim($sql_line);
            if (($tsl != "") && (substr($tsl, 0, 2) != "--") && (substr($tsl, 0, 1) != "#"))
            {
                $query .= $sql_line;
                if(preg_match('/;\s*$/', $sql_line))
                {
                    if ( $patterns != null && $replaces != null)
                    {
                        $query = preg_replace($patterns,$replaces,$query);
                    }

                    if ( !$result = mysql_query($query,$conn) )
                    {
                        switch ($onerror)
                        {
                            case 'warn':
                                $errors[] = mysql_error();
                            break;
                            case 'fail':
                                die(mysql_error());                                
                            break;
                            default:                            
                            break;
                        }
                    }
                    $query = "";               
                }
            }
        }
        
        if ( count($errors) > 0 )
        {
            return $errors;
        }
        else
        {
            return true;
        }
    }

    /*
     * Method: run
     * Run method which runs the entire installation process. This will take care of all the overhead
     * of creating the current page, capturing input and all that jazz. It then echo's the contents
     * of <buffer>.
     *
     * Parameters:
     *  noecho - Optional, toggle if page is auto echoed. This is to open up for future ajax calls.
     */
    function run( $noecho = false ){

        //establish what page the user is on//
        if ( isset($_POST['__page']) )
        {
            $this->current_page = intval($_POST['__page']);
        } 
        else 
        {
            $this->current_page = 0;
        }

        //process a submission//
        if ( !empty($_POST['__submit']) )
        {            
            //save any posted vars//
            foreach ($_POST as $key => $value)
            {
                //check for required fields//
                if ( preg_match('/^__required_/',$key) )
                {
                    if ( empty($_POST[$value]) )
                    {
                        $this->errors[$value] = 'This is a required field!';
                    }
                } 
                else 
                {
                    $this->set($key,$value);
                    if ( isset( $_POST['__validation_' . $key] ) 
                         && !preg_match($_POST['__validation_' . $key],$value) )
                    {
                        $this->errors[$key] = ($_POST['__validation_message_'.$key]?$_POST['__validation_message_'.$key]:'This field contains invalid data.');
                    }
                }
            }            

            if ( ($_POST['__submit'] == 'Next' || $_POST['__submit'] == 'Complete') 
                && count($this->errors) <= 0 )
            {
                $this->current_page++;
                $this->direction = true;
            }

            if ( $_POST['__submit'] == 'Back')
            {
                $this->current_page--;                            	
                $this->direction = false;
            }
        }

		//call the derived page function//
        $this->page($this->pages[$this->current_page]);
        
        //now take the output of the page function and inject it into the form//
        $page = '<form action="" method="post"><input type="hidden" value="'. $this->current_page .'" name="__page" />' . $this->buffer . '</form>';       

        if ( $noecho )
        {
            return $page;
        } 
        else 
        {
            echo $page;
        }
    }
}
?>