<?php
/*
 * File: Utilities
 * General simple function utilities that don't fit into any specific class.
 */
 
/*
 * Function: pick
 * This is a useful function for returning the first non-null variable.
 * 
 * Parameters:
 *  2..n - Two or more variables to look through looking for the first non-null
 *  one to return.
 * 
 * Return:
 *  mixed - The first non-null parameter or null if all parameters are null.
 * 
 * Example:
 * >function fun( $opt = null ){
 * > //looks through a couple vars before finally an empty array
 * > $opt = pick( $opt, $SOMEGLOBAL, array() );    
 * >}
 */
function pick()
{
    $args = func_get_args();
    
    foreach ( $args as $value )
    {
        if ( isset($value) && $value !== null )
        {
            return $value;
        }
    }
    
    return null;
}

/*
 * Function: bpick
 * Blank picking function for returning the first non-null and non-blank variable.
 * 
 * Parameters:
 *  2..n - Two or more variables to look through looking for the first non-null, non-blank
 *  one to return.
 * 
 * Return:
 *  mixed - The first non-null, non-blank parameter or null if all parameters are null.
 * 
 */
function bpick()
{
    $args = func_get_args();
    
    foreach ( $args as $value )
    {
        if ( isset($value) && $value !== null && $value != '' && preg_match('/\S/',$value) )
        {
            return $value;
        }
    }
    
    return null;
}

/*
 * Function: tpick
 * True picking function for returning the first variable that evaluates to a true.
 * 
 * Parameters:
 *  2..n - Two or more variables to look through looking for the first true variable
 * 
 * Return:
 *  mixed - The first true parameter or null if all parameters are null.
 * 
 */
function tpick()
{
    $args = func_get_args();
    
    foreach ( $args as $value )
    {
        if ( $value )
        {
            return $value;
        }
    }
    
    return null;
}

/*
 * Function: epick
 * Empty picking function for returning the first variable which passes !empty().
 * 
 * Parameters:
 *  2..n - Two or more variables to look through looking for the first non-empty()
 *  one to return.
 * 
 * Return:
 *  mixed - The first non-empty parameter or null if all parameters are null.
 * 
 */
function epick()
{
    $args = func_get_args();
    
    foreach ( $args as $value )
    {
        if ( !empty($value) )
        {
            return $value;
        }
    }
    
    return null;
}


/*
 * Function: imgy
 * Checks a string to see if it could possibly be an image and if so, makes an html img tag.
 * 
 * Parameters:
 *  str - String, string to test for imageification.
 *  att - Optional, associative array of attributes to add to the img tag if it's an image.
 * 
 * Return:
 *  string - Original str parameter if not an image file, img tag if str parameters is an image
 *  file.
 * 
 * Example:
 * > imgy( 'images/body.gif', array('border'=>1));
 * > -- returns --
 * > <img src="images/body.gif" border="1" />
 * >
 * > imgy( 'Edit Link', array('border'=>1));
 * > -- returns --
 * > Edit Links
 */
function imgy( $str, $att = null )
{
    if ( $str && preg_match( '/\.(gif|jpg|png|bmp|tif)$/', $str ))
    {
        $str = '<img src="' . $str . '"';
        $att = pick ( $att, array() );
        foreach ( $att as $name => $value )
        {
            $str .= ' ' . $name . '="' . htmlspecialchars($value) . '"';
        }
                
        return $str . " />";
    }
    else
    {
        return $str;
    }        
}

/*
 * Function: slash
 * Escapes a string dependent on get_magic_quotes_gpc
 * 
 * Return:
 *  string - slash escaped string
 */
function slash($str)
{
    return (get_magic_quotes_gpc()?$str:addslashes($str));
}

/*
 * Method: deslash
 * Unescapes a string dependent on get_magic_quotes_gpc
 * 
 * Return:
 *  string - string sans escape slashes
*/
function deslash($str)
{
    return (get_magic_quotes_gpc()?stripslashes($str):$str);
}

/*
 * Function: prettytext
 * Make a camelback into a normal title by injecting spaces between borders.
 * 
 * Parameters:
 *  camelcase - String to separate into spaces
 * 
 * Return:
 *  string - Titlized string
 * 
 * Example:
 *  "my_title" will return "My Title"
 *  "someOther_title" will return "Some Other Title"
 *  "OkayThenSomeThing" will return "Okay Then Some Thing"
 *  "short" will return "Short" 
 */
function prettytext( $camelcase )
{
  $pattern = array('/_+/','/\//');
  $replace = array(' ',' / ');
  $camelcase = ucwords(preg_replace($pattern,$replace,$camelcase));

  $pattern = array('/\s/','/([a-z])([A-Z0-9])/');
  $replace = array('','$1 $2');
  return preg_replace($pattern,$replace,$camelcase);
}

/*
 * Function: h123456
 * Looks through a string searching for a <h1|2|3|4|5|6> tag and returns the contents of it.
 * If none found, returns the string in default.
 * 
 * Paramters:
 *  str - String, to search through
 *  default - String, Optional contents to return if no <H> tags found.
 * 
 * Return:
 *  String - Contents of the h1 tag or default.
 */
function h123456( $str, $default = '' )
{
    if ( preg_match('/\<(h(?:1|2|3|4|5|6))\>(.*?)\<\/\1\>/i',$str,$matches) )
    {
        return $matches[2];
    } 
    else 
    {
        return $default;
    }    
}


/*
 * Function: furler
 * Looks through a string searching for a url ending in a specific extension.
 * 
 * Paramters:
 *  str - String, to search through
 *  ext - String, extensions to look for, pipe seperated, (mp3|m4a|ogg)
 *  domain - String, domain to prefix if url is missing it.
 * 
 * Return:
 *  String - Full url
 */
function furler( $str, $ext, $domain = '' )
{
    if ( preg_match('/((https?\:\/\/)?[A-Za-z0-9\.-_\/\#]+\.('. $ext .'))/i',$str,$matches) )
    {
        return ($matches[2]==''?$domain:'') . $matches[1];
    } 
    else 
    {
        return false;
    }    
}


/*
 *  Method: expand_path
 *  Takes a relative or absolute path and a context and returns an absolute path within 
 *  the galmetapost resource space. The syntax is similar but not identical to xpath's.
 *  If no proximity prefix is given ie (../,/,//,.../), then the passed path is assumed
 *  to be a child of the context, not within the same folder as common in filesystems.
 *  A (../) will reference up one resource, a / will reference an absolute, and a .../
 *  is a special case where the context is overridden by the original request. 
 * 
 *  Parameters:
 *      $path - String, absolute or relative path to expand. 
 *      $context - String, absolute path context to expand the $path within.
 *      $trinary - String, special case path for trinary dot notation (...)
 * 
 *  Returns:
 *      string - Absolute galmetapost resource path.
 *
 *  Example:
 *      expand_path ( "../x", "/my/directory" ) ==> "/x"
 *      expand_path ( "", "/my/directory" ) ==> "/my/directory"
 *      expand_Path ( "x", "/my/directory" ) ==> "/my/directory/x"
 *      expand_path ( ".../x", "/my/directory", "originalRequest" ) ==> "originalRequest/x"
 *      expand_path ( "/yours/x", "/my/directory" ) ==> "/yours/x"
 *      expand_path ( "../../../../x", "/my/directory" ) ==> "/x"
 */
function expand_path ( $path, $context, $trinary = null )
{
    $context = preg_replace('/(^\/)|(\/$)/', '', $context );

    if ( $path )
    {
        if ( $path{0} == '/' )
        {
            return substr($path,1);
        } 
        else 
        {
            //special trinary context override//
            if ( strpos( $path, '.../' ) !== false )
            {
                $context = $trinary;
                $path = str_replace('.../','',$path);
            }     
            //parental reference//           
            elseif ( strpos( $path, '../' ) !== false )
            {
                $levels = array_reverse( explode( '/', $context ) );
                preg_match( '/^((?:\.\.\/)+)(.*)$/', $path, $matches );
                $offset = strlen($matches[1]) / 3;
                $underflow = (($offset >= count($levels))?'':'/');
                return implode( '/', array_reverse( array_slice( $levels, $offset ) ) ) . $underflow . $matches[2];
            }
             
            return $context . '/' . preg_replace( '/^\.\//', '', $path );
        }
    } 
    else 
    {
        return $context;
    }
}

/*
 * Method: parse_uri
 * Sanitizes and parses a galmeta post request URI into pieces GalmetaPostResource needs
 * to set itself up. Restricts uri to 2000 characters.
 * 
 * Parameters:
 *   uri - String to parse/sanitize, the raw request
 *   context - Optional, if this resource is grouped within the context of
 *             another resource, this is that outer resource absolute uri.
 *   trinary - Optional, special case path for trinary dot operator.
 * 
 * Return:
 *   array - Associative array of the broken down, sanitized request. array(
 *      'resource' => 'full expanded resource path',
 *      'path' => 'path portion of resource, without the right-most, specific name',
 *      'name' => 'most specific id of resource, right-most',
 *      'list' => 'boolean indicating the existance of a appended slash in the case of a resource listing.',
 *      'autoid' => 'boolean indicating if this resource should be auto-id'd, when creating a resource.'
 *      'view' => 'optional view modifier following the bang in the uri',
 *      'parameters' => 'view modifier parameters, anything after view up to the end or an ampersign (&)'
 *      )
 */
function parse_uri ( $uri, $context = null, $trinary = null )
{       
    // 0 - url, 1 - absolute slash or ../'s, 2 - request, 3 - list appendix, 4 - autoid, 5 - view, 6 - params, 7 - query
    preg_match('/^((?:\/)?|(?:\.\.\.?\/)*)(?:([A-Za-z0-9][A-Za-z0-9_\/\-]*?)(\/+)?(\~)?)?(?:\!([a-z]+)([^\&]*))?(\&|$)/',$uri,$matches);
  
    if ( count($matches) > 0 )
    {
        $resource = substr( $matches[2], 0, 2000 );

        //if context passed, attempt to resolve path//
        if ( $context )
        {
            $resource = expand_path( $matches[1] . $resource . $matches[3], $context, $trinary );
            preg_match( '/^(\/)?(.*?)(\/)?$/', $resource, $subm );
            $resource = substr( $subm[2], 0, 255 );
            $matches[1] = $subm[1];
            $matches[3] = $subm[3];
        }

        //check for autoid request//
        if ( $matches[4] == '~' ){
            $resource .= '/~';
            $matches[3] = '';
        }

        //filter invalid views//
        if ( !preg_match('/'. VALID_GALMETAPOST_VIEWS .'/', $matches[5] ) )
        {
            $matches[5] = '';
            $matches[6] = '';                
        }

        $info = array(
            'resource'   => $resource,
            'path'       => ( dirname($resource) == '.' ? '/' : dirname($resource) . '/' ),
            'name'       => basename($resource),
            'list'       => (!empty($matches[3])),
            'autoid'     => (!empty($matches[4])),
            'view'       => $matches[5],
            'parameters' => substr($matches[6],0,2000)
        );
    } 
    else 
    {
        $info = array(
            'resource'   => 'page_not_found',
            'path'       => '/',
            'name'       => 'page_not_found',
            'list'       => false,
            'autoid'     => false,
            'view'       => '',
            'parameters' => ''
        );
    }

    return $info;
}

/*
 * Function: stringergy
 * Special string function which wraps $main with $pre and $post if main
 * is not blank. If $main is blank then returns $default.
 * 
 * Parameters:
 *  pre - String, prefix to return if main is not empty
 *  main - String, string to test for emptiness and return decorated with pre and post.
 *  post - String, postfix to return if main is not empty
 *  default - String, Optional, if main is empty, this will be returned. Defaults to ''.
 * 
 * Return:
 *  String - stringergized string
 */
function stringergy( $pre, $main, $post, $default = '')
{
    if ( !empty( $main ) )
    {
        $main = $pre . $main . $post;
    }
    
    return epick( $main, $default );
}

/*
 * Function: hilite
 * Special string function which hilites a string in a block of text.
 * 
 * Parameters:
 *  needle - String, text to look for and highlight
 *  text - String, block of text to look through
 *  color - String, optional, hex-code of the hightlight color
 * 
 * Return:
 *  String - block of text with highlights added
 */
function hilite( $needle, $text, $color = '#FFFFCC' )
{    
    return preg_replace( '/(' . preg_quote($needle) . ')/i', '<span class="hilite" style="background-color:'. $color .'">$1</span>', $text );
}

?>